@@ -20,8 +20,7 @@
 
 // Needed for stdio FILE
 #include <stdio.h>
-
-// Needed for time_t
+#include <stdarg.h>
 #include <time.h>
 
 #if defined(HAVE_SYS_TYPES_H)
@@ -38,14 +37,14 @@
 
 //
 // Include ImageMagick headers into namespace "MagickLib". If
-// MAGICK_IMPLEMENTATION is defined, include ImageMagick development
+// MAGICKCORE_IMPLEMENTATION is defined, include ImageMagick development
 // headers.  This scheme minimizes the possibility of conflict with
 // user code.
 //
 namespace MagickLib
 {
-#include <magick/api.h>
-#include <wand/magick-wand.h>
+#include <magick/MagickCore.h>
+#include <wand/MagickWand.h>
 #undef inline // Remove possible definition from config.h
 
 #undef class
@@ -65,7 +64,7 @@ namespace MagickLib
 #      pragma warning( disable: 4273 )  /* Disable the stupid dll linkage warnings */
 #      pragma warning( disable: 4251 )
 #    endif
-#    if !defined(MAGICK_IMPLEMENTATION)
+#    if !defined(MAGICKCORE_IMPLEMENTATION)
 #      define MagickDLLDecl __declspec(dllimport)
 #      define MagickDLLDeclExtern extern __declspec(dllimport)
 #      if defined(_VISUALC_)
@@ -105,6 +104,7 @@ namespace Magick
 {
   // The datatype for an RGB component
   using MagickLib::Quantum;
+  using MagickLib::MagickSizeType;
 
   // Boolean types
   using MagickLib::MagickBooleanType;
@@ -129,6 +129,7 @@ namespace Magick
   using MagickLib::OpacityChannel;
   using MagickLib::BlackChannel;
   using MagickLib::MatteChannel;
+  using MagickLib::DefaultChannels;
   
   // Color-space types
   using MagickLib::CMYKColorspace;
@@ -137,6 +138,7 @@ namespace Magick
   using MagickLib::HSLColorspace;
   using MagickLib::HWBColorspace;
   using MagickLib::LABColorspace;
+  using MagickLib::LogColorspace;
   using MagickLib::OHTAColorspace;
   using MagickLib::Rec601LumaColorspace;
   using MagickLib::Rec709LumaColorspace;
@@ -154,19 +156,26 @@ namespace Magick
   // Composition operations
   using MagickLib::AddCompositeOp;
   using MagickLib::AtopCompositeOp;
+  using MagickLib::BlendCompositeOp;
   using MagickLib::BumpmapCompositeOp;
   using MagickLib::ClearCompositeOp;
   using MagickLib::ColorizeCompositeOp;
   using MagickLib::CompositeOperator;
   using MagickLib::CopyBlueCompositeOp;
   using MagickLib::CopyCompositeOp;
+  using MagickLib::CopyCyanCompositeOp;
   using MagickLib::CopyGreenCompositeOp;
+  using MagickLib::CopyMagentaCompositeOp;
   using MagickLib::CopyOpacityCompositeOp;
   using MagickLib::CopyRedCompositeOp;
+  using MagickLib::CopyYellowCompositeOp;
   using MagickLib::DarkenCompositeOp;
   using MagickLib::DifferenceCompositeOp;
   using MagickLib::DisplaceCompositeOp;
   using MagickLib::DissolveCompositeOp;
+  using MagickLib::DstOverCompositeOp;
+  using MagickLib::ExclusionCompositeOp;
+  using MagickLib::HardLightCompositeOp;
   using MagickLib::HueCompositeOp;
   using MagickLib::InCompositeOp;
   using MagickLib::LightenCompositeOp;
@@ -181,13 +190,11 @@ namespace Magick
   using MagickLib::PlusCompositeOp;
   using MagickLib::SaturateCompositeOp;
   using MagickLib::ScreenCompositeOp;
+  using MagickLib::SoftLightCompositeOp;
   using MagickLib::SubtractCompositeOp;
   using MagickLib::ThresholdCompositeOp;
   using MagickLib::UndefinedCompositeOp;
   using MagickLib::XorCompositeOp;
-  using MagickLib::CopyCyanCompositeOp;
-  using MagickLib::CopyMagentaCompositeOp;
-  using MagickLib::CopyYellowCompositeOp;
   using MagickLib::CopyBlackCompositeOp;
   
   // Compression algorithms
@@ -313,6 +320,18 @@ namespace Magick
   using MagickLib::ImpulseNoise;
   using MagickLib::LaplacianNoise;
   using MagickLib::PoissonNoise;
+
+  // Orientation types
+  using MagickLib::OrientationType;
+  using MagickLib::UndefinedOrientation;
+  using MagickLib::TopLeftOrientation;
+  using MagickLib::TopRightOrientation;
+  using MagickLib::BottomRightOrientation;
+  using MagickLib::BottomLeftOrientation;
+  using MagickLib::LeftTopOrientation;
+  using MagickLib::RightTopOrientation;
+  using MagickLib::RightBottomOrientation;
+  using MagickLib::LeftBottomOrientation;
   
   // Paint methods
   using MagickLib::PaintMethod;
@@ -428,7 +447,7 @@ namespace Magick
   using MagickLib::OverlineDecoration;
   using MagickLib::LineThroughDecoration;
 
-#if defined(MAGICK_IMPLEMENTATION)
+#if defined(MAGICKCORE_IMPLEMENTATION)
   //
   // ImageMagick symbols used in implementation code
   //
@@ -458,6 +477,7 @@ namespace Magick
   using MagickLib::ChannelImage;
   using MagickLib::CharcoalImage;
   using MagickLib::ChopImage;
+  using MagickLib::ClearMagickException;
   using MagickLib::CloneDrawInfo;
   using MagickLib::CloneImage;
   using MagickLib::CloneImageInfo;
@@ -476,6 +496,7 @@ namespace Magick
   using MagickLib::ConstituteImage;
   using MagickLib::ContrastImage;
   using MagickLib::ConvolveImage;
+  using MagickLib::CopyMagickString;
   using MagickLib::CorruptImageError;
   using MagickLib::CorruptImageFatalError;
   using MagickLib::CorruptImageWarning;
@@ -616,13 +637,13 @@ namespace Magick
   using MagickLib::GetDrawInfo;
   using MagickLib::GetExceptionInfo;
   using MagickLib::GetGeometry;
-  using MagickLib::GetImageAttribute;
   using MagickLib::GetImageBoundingBox;
   using MagickLib::GetImageChannelDepth;
   using MagickLib::GetImageDepth;
   using MagickLib::GetImageInfo;
   using MagickLib::GetImageOption;
   using MagickLib::GetImagePixels;
+  using MagickLib::GetImageProperty;
   using MagickLib::GetImageProfile;
   using MagickLib::GetImageQuantizeError;
   using MagickLib::GetImageType;
@@ -634,13 +655,15 @@ namespace Magick
   using MagickLib::GetPageGeometry;
   using MagickLib::GetPixels;
   using MagickLib::GetQuantizeInfo;
+  using MagickLib::GetQuantumInfo;
+  using MagickLib::GetStringInfoDatum;
+  using MagickLib::GetStringInfoLength;
   using MagickLib::GetTypeMetrics;
   using MagickLib::GlobExpression;
   using MagickLib::GreaterValue;
   using MagickLib::HSLTransform;
   using MagickLib::HeightValue;
   using MagickLib::IdentityAffine;
-  using MagickLib::ImageAttribute;
   using MagickLib::ImageError;
   using MagickLib::ImageFatalError;
   using MagickLib::ImageInfo;
@@ -701,6 +724,7 @@ namespace Magick
   using MagickLib::ProfileInfo;
   using MagickLib::QuantizeImage;
   using MagickLib::QuantizeInfo;
+  using MagickLib::QuantumInfo;
   using MagickLib::QueryColorDatabase;
   using MagickLib::RGBTransformImage;
   using MagickLib::RaiseImage;
@@ -715,6 +739,7 @@ namespace Magick
   using MagickLib::RegistryWarning;
   using MagickLib::RelinquishMagickMemory;
   using MagickLib::RemoveImageOption;
+  using MagickLib::ResizeImage;
   using MagickLib::ResizeMagickMemory;
   using MagickLib::ResourceLimitError;
   using MagickLib::ResourceLimitFatalError;
@@ -728,7 +753,6 @@ namespace Magick
   using MagickLib::SetCacheView;
   using MagickLib::SetClientName;
   using MagickLib::SetImage;
-  using MagickLib::SetImageAttribute;
   using MagickLib::SetImageChannelDepth;
   using MagickLib::SetImageClipMask;
   using MagickLib::SetImageDepth;
@@ -737,6 +761,7 @@ namespace Magick
   using MagickLib::SetImageOption;
   using MagickLib::SetImagePixels;
   using MagickLib::SetImageProfile;
+  using MagickLib::SetImageProperty;
   using MagickLib::SetImageType;
   using MagickLib::SetLogEventMask;
   using MagickLib::SetMagickInfo;
@@ -746,6 +771,7 @@ namespace Magick
   using MagickLib::SharpenImage;
   using MagickLib::ShaveImage;
   using MagickLib::ShearImage;
+  using MagickLib::SigmoidalContrastImageChannel;
   using MagickLib::SignatureImage;
   using MagickLib::SizeBlob;
   using MagickLib::SolarizeImage;
@@ -767,6 +793,7 @@ namespace Magick
   using MagickLib::TransformImage;
   using MagickLib::TransformRGBImage;
   using MagickLib::TransparentImage;
+  using MagickLib::TrimImage;
   using MagickLib::TypeError;
   using MagickLib::TypeFatalError;
   using MagickLib::TypeWarning;
@@ -787,7 +814,7 @@ namespace Magick
   using MagickLib::YValue;
   using MagickLib::ZoomImage;
 
-#endif // MAGICK_IMPLEMENTATION
+#endif // MAGICKCORE_IMPLEMENTATION
 
 }
 
