@@ -447,10 +447,10 @@ int main( int /*argc*/, char ** argv)
     //
     // classType
     //
-    if ( image.classType() != DirectClass )
+    if ( image.classType() != PseudoClass )
       {
 	++failures;
-	cout << "Line: " << __LINE__ << ", classType is not DirectClass" << endl;
+	cout << "Line: " << __LINE__ << ", classType is not PseudoClass" << endl;
       }
 
     //
@@ -740,13 +740,13 @@ int main( int /*argc*/, char ** argv)
     // fontPointsize
     //
     // Test default
-    if ( image.fontPointsize() != 12 )
+    if ( image.fontPointsize() != 0 )
       {
 	++failures;
 	cout << "Line: " << __LINE__
              << ", fontPointsize ("
              << image.fontPointsize()
-             << ") is not default of 12 as expected"
+             << ") is not default of 0 as expected"
              << endl;
       }
 
@@ -913,11 +913,11 @@ int main( int /*argc*/, char ** argv)
     // matte
     //
     // Test default
-    if ( image.matte() == false )
+    if ( image.matte() != false )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", matte default is not true as expected" << endl;
+             << ", matte default is not false as expected" << endl;
       }
 
     // Test set/get
@@ -1151,7 +1151,7 @@ int main( int /*argc*/, char ** argv)
     // quantizeColorSpace
     //
     // Test default
-    if ( image.quantizeColorSpace() != RGBColorspace )
+    if ( image.quantizeColorSpace() != UndefinedColorspace )
       {
 	++failures;
 	cout << "Line: " << __LINE__
@@ -1284,7 +1284,7 @@ int main( int /*argc*/, char ** argv)
 
     if ( image.signature() != "849077c0eb67c77cf3b06ff2419cc2b11dea5ff7d8f95466914f23731d1f1a68" &&
 	 image.signature() != "7fe11319c31622db67434ab155cd733afbb391e5dbc2657d08893ebea19ac6dc" &&
-         image.signature() != "56f66a46543e65e7f78022c1c5cb3bd76b82b987b0daaf5cb33fab12caf0563f")
+         image.signature() != "9e8b6d6d8036e9346d38a26f55e64a695b61ee9b9234e61198f9c497db392c6b")
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", signature ("
