@@ -286,6 +286,20 @@ static const unsigned char MJPGDHTSeg[0x1A8] = {
 %
 */
 
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType EmitMessage(j_common_ptr jpeg_info,int level)
 {
   char
@@ -324,14 +338,14 @@ static boolean FillInputBuffer(j_decompress_ptr cinfo)
   source=(SourceManager *) cinfo->src;
   if (source->image)
     source->manager.bytes_in_buffer=(size_t) ReadBlob(source->image,
-      Min(source->source_length,MaxBufferExtent),source->buffer);
+      MagickMin(source->source_length,MaxBufferExtent),source->buffer);
   else
     {
-      source->manager.bytes_in_buffer=(size_t)
-        Min(source->source_length,MaxBufferExtent);
+      source->manager.bytes_in_buffer=(size_t) MagickMin(source->source_length,
+        MaxBufferExtent);
       (void) CopyMagickMemory(source->buffer,source->source_data,
         source->manager.bytes_in_buffer);
-      source->source_data += source->manager.bytes_in_buffer;
+      source->source_data+=source->manager.bytes_in_buffer;
     }
   source->source_length-=source->manager.bytes_in_buffer;
   if (source->manager.bytes_in_buffer == 0)
@@ -480,8 +494,8 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   if (profile == (StringInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  p=profile->datum;
-  for (i=(long) profile->length-1; i >= 0; i--)
+  p=GetStringInfoDatum(profile);
+  for (i=(long) GetStringInfoLength(profile)-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
   icc_profile=(StringInfo *) GetImageProfile(image,"icc");
   if (icc_profile != (StringInfo *) NULL)
@@ -598,8 +612,8 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   if (profile == (StringInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  p=profile->datum;
-  for (i=(long) profile->length-1; i >= 0; i--)
+  p=GetStringInfoDatum(profile);
+  for (i=(long) GetStringInfoLength(profile)-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
   iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
   if (iptc_profile != (StringInfo *) NULL)
@@ -662,12 +676,12 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
   if (profile == (StringInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  p=profile->datum;
-  for (i=(long) profile->length-1; i >= 0; i--)
+  p=GetStringInfoDatum(profile);
+  for (i=(long) GetStringInfoLength(profile)-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
   if (marker == 1)
     {
-      p=profile->datum;
+      p=GetStringInfoDatum(profile);
       if ((length > 4) && (LocaleNCompare((char *) p,"exif",4) == 0))
         (void) CopyMagickString(name,"iptc",MaxTextExtent);
       if ((length > 5) && (LocaleNCompare((char *) p,"http:",5) == 0))
@@ -832,7 +846,7 @@ static MagickBooleanType DecodeImage(Image *image,
       break;
     if (count != 0)
       {
-        count=Min(count,(ssize_t) (q-p));
+        count=(ssize_t) MagickMin(count,(size_t) (q-p));
         /*
           Encoded mode.
         */
@@ -885,7 +899,7 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            count=Min(count,(ssize_t) (q-p));
+            count=(long) MagickMin(count,(size_t) (q-p));
             if (compression == BI_RLE8)
               for (i=0; i < (long) count; i++)
                 *p++=(unsigned char) ReadBlobByte(image);
@@ -1635,13 +1649,12 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (image->scene >= (image_info->scene+image_info->number_scenes-1))
               break;
           bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
-          pixels=(unsigned char *) AcquireMagickMemory((size_t)
-            Max(bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
+          pixels=(unsigned char *) AcquireMagickMemory((size_t) MagickMax(
+            bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
           if (pixels == (unsigned char *) NULL)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-          (void) ResetMagickMemory(pixels,0,(size_t)
-            Max(bytes_per_line,image->columns+1)*image->rows);
-
+          (void) ResetMagickMemory(pixels,0,(size_t)  MagickMax(
+            bytes_per_line,image->columns+1)*image->rows);
           if (LocaleCompare(id,"00db") == 0)
             {
               count=ReadBlob(image,(size_t) bytes_per_line*image->rows,pixels);
