@@ -165,6 +165,27 @@ static MagickBooleanType
 %
 %
 */
+static inline long MagickAbsoluteValue(const long x)
+{
+  if (x < 0)
+    return(-x);
+  return(x);
+}
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline long MagickMin(const long x,const long y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType DecodeImage(Image *image,
   const unsigned long compression,unsigned char *pixels)
 {
@@ -211,7 +232,7 @@ static MagickBooleanType DecodeImage(Image *image,
         /*
           Encoded mode.
         */
-        count=Min(count,(int) (q-p));
+        count=MagickMin(count,(int) (q-p));
         byte=(unsigned char) ReadBlobByte(image);
         if (compression == BI_RLE8)
           {
@@ -261,7 +282,7 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            count=Min(count,(int) (q-p));
+            count=MagickMin(count,(int) (q-p));
             if (compression == BI_RLE8)
               for (i=0; i < count; i++)
                 *p++=(unsigned char) ReadBlobByte(image);
@@ -470,8 +491,8 @@ static MagickBooleanType IsBMP(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
 static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   BMPInfo
@@ -808,7 +829,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ThrowReaderException(CorruptImageError,"UnrecognizedImageCompression");
     }
     image->columns=(unsigned long) bmp_info.width;
-    image->rows=(unsigned long) AbsoluteValue(bmp_info.height);
+    image->rows=(unsigned long) MagickAbsoluteValue(bmp_info.height);
     image->depth=8;
     image->matte=bmp_info.alpha_mask != 0 ? MagickTrue : MagickFalse;
     if ((bmp_info.number_colors != 0) || (bmp_info.bits_per_pixel < 16))
@@ -868,8 +889,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       bmp_info.bits_per_pixel<<=1;
     bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
     length=(size_t) bytes_per_line*image->rows;
-    pixels=(unsigned char *) AcquireMagickMemory((size_t)
-      Max(bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
+    pixels=(unsigned char *) AcquireMagickMemory((size_t) MagickMax(
+      bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
     if (pixels == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((bmp_info.compression == BI_RGB) ||
@@ -897,6 +918,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     image->x_resolution=(double) bmp_info.x_pixels/100.0;
     image->y_resolution=(double) bmp_info.y_pixels/100.0;
+    image->units=PixelsPerCentimeterResolution;
     /*
       Convert BMP raster image to pixel packets.
     */
@@ -1968,7 +1990,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
         if (bmp_colormap == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         q=bmp_colormap;
-        for (i=0; i < (long) Min(image->colors,bmp_info.number_colors); i++)
+        for (i=0; i < (long) MagickMin(image->colors,bmp_info.number_colors); i++)
         {
           *q++=ScaleQuantumToChar(image->colormap[i].blue);
           *q++=ScaleQuantumToChar(image->colormap[i].green);
