@@ -134,8 +134,15 @@ static MagickBooleanType
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the decoding process.
 %
-%
 */
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType DecodeImage(Image *image,
   const MagickBooleanType compression,unsigned char *pixels)
 {
@@ -186,7 +193,7 @@ static MagickBooleanType DecodeImage(Image *image,
       break;
     if (count != 0)
       {
-        count=(int) Min((unsigned long) count,(size_t) (q-p));
+        count=(int) MagickMin(count,(size_t) (q-p));
         /*
           Encoded mode.
         */
@@ -239,7 +246,7 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            count=(int) Min((unsigned long) count,(size_t) (q-p));
+            count=(int) MagickMin(count,(size_t) (q-p));
             if (compression == BI_RLE8)
               for (i=0; i < count; i++)
                 *p++=(unsigned char) ReadBlobByte(image);
@@ -442,8 +449,22 @@ static MagickBooleanType IsDIB(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline long MagickAbsoluteValue(const long x)
+{
+  if (x < 0)
+    return(-x);
+  return(x);
+}
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   DIBInfo
@@ -535,7 +556,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   image->matte=dib_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
   image->columns=(unsigned long) dib_info.width;
-  image->rows=(unsigned long) AbsoluteValue(dib_info.height);
+  image->rows=(unsigned long) MagickAbsoluteValue(dib_info.height);
   image->depth=8;
   if ((dib_info.number_colors != 0) || (dib_info.bits_per_pixel < 16))
     {
@@ -598,8 +619,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     dib_info.bits_per_pixel<<=1;
   bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
   length=bytes_per_line*image->rows;
-  pixels=(unsigned char *) AcquireMagickMemory((size_t)
-    Max(bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
+  pixels=(unsigned char *) AcquireMagickMemory((size_t) MagickMax(
+    bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if ((dib_info.compression == BI_RGB) ||
@@ -1272,7 +1293,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
           if (dib_colormap == (unsigned char *) NULL)
             ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
           q=dib_colormap;
-          for (i=0; i < (long) Min(image->colors,dib_info.number_colors); i++)
+          for (i=0; i < (long) MagickMin(image->colors,dib_info.number_colors); i++)
           {
             *q++=ScaleQuantumToChar(image->colormap[i].blue);
             *q++=ScaleQuantumToChar(image->colormap[i].green);
