@@ -157,8 +157,15 @@ ModuleExport void UnregisterHISTOGRAMImage(void)
 %
 %    o image:  The image.
 %
-%
 */
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   Image *image)
 {
@@ -235,7 +242,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   /*
     Allocate histogram count arrays.
   */
-  length=(size_t) Max((unsigned long) ScaleQuantumToChar(QuantumRange)+1,
+  length=MagickMax((size_t) ScaleQuantumToChar(QuantumRange)+1,
     histogram_image->columns);
   histogram=(MagickPixelPacket *)
     AcquireMagickMemory(length*sizeof(*histogram));
@@ -342,15 +349,19 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   if ((unique_file != -1) && (file != (FILE *) NULL))
     {
       char
-        command[MaxTextExtent];
+        *property;
 
       /*
         Add a histogram as an image comment.
       */
       (void) GetNumberColors(image,file,&image->exception);
       (void) fclose(file);
-      (void) FormatMagickString(command,MaxTextExtent,"@%s",filename);
-      (void) SetImageProperty(histogram_image,"Comment",command);
+      property=FileToString(filename,~0,&image->exception);
+      if (property != (char *) NULL)
+        {
+          (void) SetImageProperty(histogram_image,"Comment",property);
+          property=(char *) RelinquishMagickMemory(property);
+        }
     }
   (void) RelinquishUniqueFileResource(filename);
   /*
@@ -362,7 +373,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   (void) SetImageInfo(write_info,MagickTrue,&image->exception);
   if (LocaleCompare(write_info->magick,"HISTOGRAM") == 0)
     (void) FormatMagickString(histogram_image->filename,MaxTextExtent,
-      "miff:%s",image_info->filename);
+      "miff:%s",write_info->filename);
   status=WriteImage(write_info,histogram_image);
   histogram_image=DestroyImage(histogram_image);
   write_info=DestroyImageInfo(write_info);
