@@ -79,7 +79,7 @@ typedef struct _IconEntry
     colors,
     reserved;
 
-  short int
+  unsigned short int
     planes,
     bits_per_pixel;
 
@@ -203,6 +203,10 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     bit,
     byte;
 
+  unsigned long
+    bytes_per_line,
+    scanline_pad;
+
   /*
     Open image file.
   */
@@ -231,8 +235,9 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     icon_file.directory[i].height=(unsigned char) ReadBlobByte(image);
     icon_file.directory[i].colors=(unsigned char) ReadBlobByte(image);
     icon_file.directory[i].reserved=(unsigned char) ReadBlobByte(image);
-    icon_file.directory[i].planes=(short) ReadBlobLSBShort(image);
-    icon_file.directory[i].bits_per_pixel=(short) ReadBlobLSBShort(image);
+    icon_file.directory[i].planes=(unsigned short) ReadBlobLSBShort(image);
+    icon_file.directory[i].bits_per_pixel=(unsigned short)
+      ReadBlobLSBShort(image);
     icon_file.directory[i].size=ReadBlobLSBLong(image);
     icon_file.directory[i].offset=ReadBlobLSBLong(image);
   }
@@ -244,8 +249,8 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     (void) SeekBlob(image,(MagickOffsetType) icon_file.directory[i].offset,
       SEEK_SET);
     icon_info.size=ReadBlobLSBLong(image);
-    icon_info.width=ReadBlobLSBLong(image);
-    icon_info.height=ReadBlobLSBLong(image);
+    icon_info.width=(unsigned char) ReadBlobLSBLong(image);
+    icon_info.height=(unsigned char) ReadBlobLSBLong(image)/2;
     icon_info.planes=ReadBlobLSBShort(image);
     icon_info.bits_per_pixel=ReadBlobLSBShort(image);
     icon_info.compression=ReadBlobLSBLong(image);
@@ -254,9 +259,9 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     icon_info.y_pixels=ReadBlobLSBLong(image);
     icon_info.number_colors=ReadBlobLSBLong(image);
     icon_info.colors_important=ReadBlobLSBLong(image);
-    image->matte=icon_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
-    image->columns=icon_info.width;
-    image->rows=icon_info.height;
+    image->matte=MagickTrue;
+    image->columns=(unsigned long) icon_info.width;
+    image->rows=(unsigned long) icon_info.height;
     image->depth=8;
     if (image->debug != MagickFalse)
     {
@@ -323,6 +328,9 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
+    bytes_per_line=(((image->columns*icon_info.bits_per_pixel)+31) & ~31) >> 3;
+    scanline_pad=((((image->columns*icon_info.bits_per_pixel)+31) & ~31)-
+      (image->columns*icon_info.bits_per_pixel)) >> 3;
     switch (icon_info.bits_per_pixel)
     {
       case 1:
@@ -350,6 +358,8 @@ static Image *ReadICONImage(const ImageInfo *image_info,
                 indexes[x+bit]=(IndexPacket)
                   ((byte & (0x80 >> bit)) != 0 ? 0x01 : 0x00);
             }
+          for (x=0; x < (long) scanline_pad; x++)
+            (void) ReadBlobByte(image);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -386,13 +396,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
               byte=(unsigned long) ReadBlobByte(image);
               indexes[x]=(IndexPacket) ((byte >> 4) & 0xf);
             }
+          for (x=0; x < (long) scanline_pad; x++)
+            (void) ReadBlobByte(image);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
                   image->rows,image->client_data);
                 if (status == MagickFalse)
                   break;
@@ -416,13 +428,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             byte=(unsigned long) ReadBlobByte(image);
             indexes[x]=(IndexPacket) byte;
           }
+          for (x=0; x < (long) scanline_pad; x++)
+            (void) ReadBlobByte(image);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
                   image->rows,image->client_data);
                 if (status == MagickFalse)
                   break;
@@ -448,13 +462,15 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             byte=(unsigned long) ReadBlobByte(image);
             indexes[x]|=byte << 8;
           }
+          for (x=0; x < (long) scanline_pad; x++)
+            (void) ReadBlobByte(image);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
                   image->rows,image->client_data);
                 if (status == MagickFalse)
                   break;
@@ -478,18 +494,21 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             q->blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             q->green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             q->red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            if (image->matte != MagickFalse)
+            if (icon_info.bits_per_pixel == 32)
               q->opacity=QuantumRange-ScaleCharToQuantum((unsigned char)
                 ReadBlobByte(image));
             q++;
           }
+          if (icon_info.bits_per_pixel == 24)
+            for (x=0; x < (long) scanline_pad; x++)
+              (void) ReadBlobByte(image);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
                   image->rows,image->client_data);
                 if (status == MagickFalse)
                   break;
@@ -501,39 +520,34 @@ static Image *ReadICONImage(const ImageInfo *image_info,
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
     (void) SyncImage(image);
-    if (image->matte != MagickTrue)
+
+    image->storage_class=DirectClass;
+    image->matte=MagickTrue;
+    for (y=(long) image->rows-1; y >= 0; y--)
+    {
+      q=GetImagePixels(image,0,y,image->columns,1);
+      if (q == (PixelPacket *) NULL)
+        break;
+      for (x=0; x < ((long) image->columns-7); x+=8)
       {
-        /*
-          Convert bitmap scanline to pixel packets.
-        */
-        image->storage_class=DirectClass;
-        image->matte=MagickTrue;
-        for (y=(long) image->rows-1; y >= 0; y--)
-        {
-          q=GetImagePixels(image,0,y,image->columns,1);
-          if (q == (PixelPacket *) NULL)
-            break;
-          for (x=0; x < ((long) image->columns-7); x+=8)
-          {
-            byte=(unsigned long) ReadBlobByte(image);
-            for (bit=0; bit < 8; bit++)
-              q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != 0) ?
-                TransparentOpacity : OpaqueOpacity);
-          }
-          if ((image->columns % 8) != 0)
-            {
-              byte=(unsigned long) ReadBlobByte(image);
-              for (bit=0; bit < (image->columns % 8); bit++)
-                q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != 0) ?
-                  TransparentOpacity : OpaqueOpacity);
-            }
-         if ((image->columns % 32)  != 0)
-           for (x=0; x < (long) ((32-(image->columns % 32))/8); x++)
-             (void) ReadBlobByte(image);
-          if (SyncImagePixels(image) == MagickFalse)
-            break;
-        }
+        byte=(unsigned long) ReadBlobByte(image);
+        for (bit=0; bit < 8; bit++)
+          q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != 0) ?
+            TransparentOpacity : OpaqueOpacity);
       }
+      if ((image->columns % 8) != 0)
+        {
+          byte=(unsigned long) ReadBlobByte(image);
+          for (bit=0; bit < (image->columns % 8); bit++)
+            q[x+bit].opacity=(Quantum) (((byte & (0x80 >> bit)) != 0) ?
+              TransparentOpacity : OpaqueOpacity);
+        }
+     if ((image->columns % 32)  != 0)
+       for (x=0; x < (long) ((32-(image->columns % 32))/8); x++)
+         (void) ReadBlobByte(image);
+      if (SyncImagePixels(image) == MagickFalse)
+        break;
+    }
     if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
@@ -677,7 +691,6 @@ ModuleExport void UnregisterICONImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   Image *image)
@@ -814,20 +827,16 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
             icon_info.offset_bits+=(1UL << icon_info.bits_per_pixel);
           }
       }
-    bytes_per_line=4*((image->columns*icon_info.bits_per_pixel+31)/32);
+    bytes_per_line=(((image->columns*icon_info.bits_per_pixel)+31) & ~31) >> 3;
     icon_info.ba_offset=0;
-    if (image->matte == MagickFalse)
-      icon_info.size=40;
-    else
-      {
-        icon_info.size=108;
-        icon_info.file_size+=68;
-        icon_info.offset_bits+=68;
-      }
     icon_info.width=(long) image->columns;
     icon_info.height=(long) image->rows;
     icon_info.planes=1;
     icon_info.image_size=bytes_per_line*image->rows;
+    icon_info.size=40;
+    icon_info.size+=(4*icon_info.number_colors);
+    icon_info.size+=icon_info.image_size;
+    icon_info.size+=(((icon_info.width+31) & ~31) >> 3)*icon_info.height;
     icon_info.file_size+=icon_info.image_size;
     icon_info.x_pixels=75*39;
     icon_info.y_pixels=75*39;
@@ -1011,16 +1020,16 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     /*
       Write 40-byte version 3+ bitmap header.
     */
-    icon_file.directory[scene].width=icon_info.width;
-    icon_file.directory[scene].height=icon_info.height;
-    icon_file.directory[scene].colors=icon_info.number_colors;
+    icon_file.directory[scene].width=(unsigned char) icon_info.width;
+    icon_file.directory[scene].height=(unsigned char) icon_info.height;
+    icon_file.directory[scene].colors=(unsigned char) icon_info.number_colors;
     icon_file.directory[scene].planes=icon_info.planes;
     icon_file.directory[scene].bits_per_pixel=icon_info.bits_per_pixel;
-    icon_file.directory[scene].size=icon_info.image_size;
-    icon_file.directory[scene].offset=TellBlob(image);
-    (void) WriteBlobLSBLong(image,icon_info.size);
+    icon_file.directory[scene].size=icon_info.size;
+    icon_file.directory[scene].offset=(unsigned long) TellBlob(image);
+    (void) WriteBlobLSBLong(image,(unsigned long) 40);
     (void) WriteBlobLSBLong(image,(unsigned long) icon_info.width);
-    (void) WriteBlobLSBLong(image,(unsigned long) icon_info.height);
+    (void) WriteBlobLSBLong(image,(unsigned long) icon_info.height * 2);
     (void) WriteBlobLSBShort(image,icon_info.planes);
     (void) WriteBlobLSBShort(image,icon_info.bits_per_pixel);
     (void) WriteBlobLSBLong(image,icon_info.compression);
@@ -1065,8 +1074,8 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     /*
       Write matte mask.
     */
-    scanline_pad=(((image->columns+8-1)/8+0x03) & ~0x03)-(image->columns/8);
-    for (y=0; y < (long) image->rows; y++)
+    scanline_pad=(((image->columns+31) & ~31)-image->columns) >> 3;
+    for (y=((long) image->rows - 1); y >= 0; y--)
     {
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
@@ -1075,9 +1084,9 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
       byte=0;
       for (x=0; x < (long) image->columns; x++)
       {
-        byte>>=1;
-        if ((image->matte == MagickFalse) || (p->opacity != OpaqueOpacity))
-          byte|=0x80;
+        byte<<=1;
+        if ((image->matte == MagickTrue) && (p->opacity == TransparentOpacity))
+          byte|=0x01;
         bit++;
         if (bit == 8)
           {
@@ -1088,9 +1097,9 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
         p++;
       }
       if (bit != 0)
-        (void) WriteBlobByte(image,(unsigned char) (byte >> (8-bit)));
+        (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
       for (i=0; i < (long) scanline_pad; i++)
-        (void) WriteBlobByte(image,(unsigned char) (byte >> (8-bit)));
+        (void) WriteBlobByte(image,(unsigned char) 0);
     }
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
@@ -1118,8 +1127,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     (void) WriteBlobByte(image,icon_file.directory[scene].reserved);
     (void) WriteBlobLSBShort(image,icon_file.directory[scene].planes);
     (void) WriteBlobLSBShort(image,icon_file.directory[scene].bits_per_pixel);
-    (void) WriteBlobLSBLong(image,icon_file.directory[scene].size+
-      4*icon_file.directory[scene].colors+40);
+    (void) WriteBlobLSBLong(image,icon_file.directory[scene].size);
     (void) WriteBlobLSBLong(image,icon_file.directory[scene].offset);
     scene++;
     next=SyncNextImageInList(next);
