@@ -396,9 +396,6 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   double
     version;
 
-  IndexPacket
-    polarity;
-
   long
     y;
 
@@ -459,13 +456,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
     /*
       Convert pixels to a bitmap.
     */
-    (void) SetImageType(image,PaletteType);
     (void) SetImageType(image,BilevelType);
-    polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
-      (QuantumRange/2) ? 1 : 0);
-    if (image->colors == 2)
-      polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) >
-        PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
     q=pixels;
     for (y=0; y < (long) image->rows; y++)
     {
@@ -478,7 +469,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
       for (x=0; x < (long) image->columns; x++)
       {
         byte<<=1;
-        if (indexes[x] == polarity)
+        if (PixelIntensity(p) < (QuantumRange/2.0))
           byte|=0x01;
         bit++;
         if (bit == 8)
@@ -487,7 +478,8 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
             bit=0;
             byte=0;
           }
-       }
+        p++;
+      }
       if (bit != 0)
         *q++=byte << (8-bit);
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
