@@ -386,8 +386,8 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   if (profile == (StringInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  p=profile->datum;
-  for (i=(long) profile->length-1; i >= 0; i--)
+  p=GetStringInfoDatum(profile);
+  for (i=(long) GetStringInfoLength(profile)-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
   icc_profile=(StringInfo *) GetImageProfile(image,"icc");
   if (icc_profile != (StringInfo *) NULL)
@@ -448,7 +448,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   tag_length=0;
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
-  iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
+  iptc_profile=(StringInfo *) GetImageProfile(image,"8bim");
   if (iptc_profile == (StringInfo *) NULL)
     {
       /*
@@ -485,10 +485,10 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   if (profile == (StringInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  p=profile->datum;
-  for (i=(long) profile->length-1; i >= 0; i--)
+  p=GetStringInfoDatum(profile);
+  for (i=(long) GetStringInfoLength(profile)-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
-  iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
+  iptc_profile=(StringInfo *) GetImageProfile(image,"8bim");
   if (iptc_profile != (StringInfo *) NULL)
     {
       ConcatenateStringInfo(iptc_profile,profile);
@@ -496,7 +496,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
     }
   else
     {
-      status=SetImageProfile(image,"iptc",profile);
+      status=SetImageProfile(image,"8bim",profile);
       profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
@@ -553,12 +553,12 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
   if (profile == (StringInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  p=profile->datum;
-  for (i=(long) profile->length-1; i >= 0; i--)
+  p=GetStringInfoDatum(profile);
+  for (i=(long) GetStringInfoLength(profile)-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
   if (marker == 1)
     {
-      p=profile->datum;
+      p=GetStringInfoDatum(profile);
       if ((length > 4) && (LocaleNCompare((char *) p,"exif",4) == 0))
         (void) CopyMagickString(name,"exif",MaxTextExtent);
       if ((length > 5) && (LocaleNCompare((char *) p,"http:",5) == 0))
@@ -961,7 +961,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   }
   (void) FormatMagickString(value,MaxTextExtent,"%ld",(long)
     jpeg_info.out_color_space);
-  (void) SetImageProperty(image,"JPEG-Colorspace",value);
+  (void) SetImageProperty(image,"jpeg:colorspace",value);
   /*
     Set image sampling factors.
   */
@@ -1018,7 +1018,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       break;
     }
   }
-  (void) SetImageProperty(image,"JPEG-Sampling-factors",sampling_factors);
+  (void) SetImageProperty(image,"jpeg:sampling-factor",sampling_factors);
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Sampling Factors: %s",sampling_factors);
   if (image_info->ping != MagickFalse)
@@ -1262,6 +1262,7 @@ ModuleExport void RegisterJPEGImage(void)
 */
 ModuleExport void UnregisterJPEGImage(void)
 {
+  (void) UnregisterMagickInfo("PJPG");
   (void) UnregisterMagickInfo("JPEG");
   (void) UnregisterMagickInfo("JPG");
 }
@@ -1321,6 +1322,13 @@ static void InitializeDestination(j_compress_ptr cinfo)
   destination->manager.free_in_buffer=MaxBufferExtent;
 }
 
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static void TerminateDestination(j_compress_ptr cinfo)
 {
   DestinationManager
@@ -1371,60 +1379,68 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
   {
     profile=GetImageProfile(image,name);
     if ((LocaleCompare(name,"EXIF") == 0) || (LocaleCompare(name,"XMP") == 0))
-      for (i=0; i < (long) profile->length; i+=65533L)
+      for (i=0; i < (long) GetStringInfoLength(profile); i+=65533L)
       {
-        length=Min(profile->length-i,65533L);
-        jpeg_write_marker(jpeg_info,XML_MARKER,profile->datum+i,
+        length=MagickMin(GetStringInfoLength(profile)-i,65533L);
+        jpeg_write_marker(jpeg_info,XML_MARKER,GetStringInfoDatum(profile)+i,
           (unsigned int) length);
       }
     if (LocaleCompare(name,"ICC") == 0)
       {
+        register unsigned char
+          *p;
+
         tag_length=14;
-        (void) CopyMagickMemory(custom_profile->datum,ICC_PROFILE,tag_length);
-        for (i=0; i < (long) profile->length; i+=65519L)
+        p=GetStringInfoDatum(custom_profile);
+        (void) CopyMagickMemory(p,ICC_PROFILE,tag_length);
+        for (i=0; i < (long) GetStringInfoLength(profile); i+=65519L)
         {
-          length=Min(profile->length-i,65519L);
-          custom_profile->datum[12]=(unsigned char) ((i/65519L)+1);
-          custom_profile->datum[13]=(unsigned char) (profile->length/65519L+1);
-          (void) CopyMagickMemory(custom_profile->datum+tag_length,
-            profile->datum+i,length);
-          jpeg_write_marker(jpeg_info,ICC_MARKER,custom_profile->datum,
-            (unsigned int) (length+tag_length));
+          length=MagickMin(GetStringInfoLength(profile)-i,65519L);
+          p=GetStringInfoDatum(custom_profile);
+          p[12]=(unsigned char) ((i/65519L)+1);
+          p[13]=(unsigned char) (GetStringInfoLength(profile)/65519L+1);
+          (void) CopyMagickMemory(p+tag_length,GetStringInfoDatum(profile)+i,
+            length);
+          jpeg_write_marker(jpeg_info,ICC_MARKER,GetStringInfoDatum(
+            custom_profile),(unsigned int) (length+tag_length));
         }
       }
     if (LocaleCompare(name,"IPTC") == 0)
       {
+        register unsigned char
+          *p;
+
         unsigned long
           roundup;
 
-        if (LocaleNCompare((char *) profile->datum,"8BIM",4) == 0)
+        p=GetStringInfoDatum(custom_profile);
+        if (LocaleNCompare((char *) GetStringInfoDatum(profile),"8BIM",4) == 0)
           {
-            (void) CopyMagickMemory(custom_profile->datum,"Photoshop 3.0\0",14);
+            (void) CopyMagickMemory(p,"Photoshop 3.0\0",14);
             tag_length=14;
           }
         else
           {
-            (void) CopyMagickMemory(custom_profile->datum,
-              "Photoshop 3.0\08BIM\04\04\0\0\0\0",24);
-            custom_profile->datum[13]=0x00;
-            custom_profile->datum[24]=(unsigned char) (profile->length >> 8);
-            custom_profile->datum[25]=(unsigned char) (profile->length & 0xff);
+            (void) CopyMagickMemory(p,"Photoshop 3.0\08BIM\04\04\0\0\0\0",24);
+            p[13]=0x00;
+            p[24]=(unsigned char) (GetStringInfoLength(profile) >> 8);
+            p[25]=(unsigned char) (GetStringInfoLength(profile) & 0xff);
             tag_length=26;
           }
-        for (i=0; i < (long) profile->length; i+=65500L)
+        for (i=0; i < (long) GetStringInfoLength(profile); i+=65500L)
         {
-          length=Min(profile->length-i,65500L);
+          length=MagickMin(GetStringInfoLength(profile)-i,65500L);
           roundup=(unsigned long) (length & 0x01);
-          (void) CopyMagickMemory(custom_profile->datum+tag_length,
-            profile->datum+i,length);
+          (void) CopyMagickMemory(p+tag_length,GetStringInfoDatum(profile)+i,
+            length);
           if (roundup != 0)
-            custom_profile->datum[length+tag_length]='\0';
-          jpeg_write_marker(jpeg_info,IPTC_MARKER,custom_profile->datum,
-            (unsigned int) (length+tag_length+roundup));
+            p[length+tag_length]='\0';
+          jpeg_write_marker(jpeg_info,IPTC_MARKER,GetStringInfoDatum(
+            custom_profile),(unsigned int) (length+tag_length+roundup));
         }
       }
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-      "%s profile: %lu bytes",name,(unsigned long) profile->length);
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s profile: %lu bytes",
+      name,(unsigned long) GetStringInfoLength(profile));
     name=GetNextImageProfile(image);
   }
   custom_profile=DestroyStringInfo(custom_profile);
@@ -1731,7 +1747,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
 #endif
     }
   sampling_factors=(const char *) NULL;
-  value=GetImageProperty(image,"JPEG-Sampling-factors");
+  value=GetImageProperty(image,"jpeg:sampling-factor");
   if (value != (char *) NULL)
     {
       sampling_factors=value;
@@ -1918,7 +1934,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   if (value != (char *) NULL)
     for (i=0; i < (long) strlen(value); i+=65533L)
       jpeg_write_marker(&jpeg_info,JPEG_COM,(unsigned char *) value+i,
-        (unsigned int) Min((long) strlen(value+i),65533L));
+        (unsigned int) MagickMin((long) strlen(value+i),65533L));
   if (image->profiles != (void *) NULL)
     WriteProfile(&jpeg_info,image);
   /*
