@@ -394,6 +394,13 @@ static void InsertWordRow(Image *image,int channel,unsigned char *p,int y)
   return;
 }
 
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static void ReadBlobWordLSB(Image *image,size_t len,unsigned short *data)
 {
   while (len >= 2)
@@ -549,7 +556,7 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     case 2:
     {
-      image->depth=Min(QuantumDepth,8);         /*Byte type cell */
+      image->depth=(unsigned long) MagickMin(QuantumDepth,8);         /*Byte type cell */
       ldblk=(long) mat_header.SizeX;
       if ((mat_header.StructureFlag && FLAG_COMPLEX) != 0)
         goto MATLAB_KO;
@@ -557,7 +564,7 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case 4:
     {
-      image->depth=Min(QuantumDepth,16);        /*Word type cell */
+      image->depth=(unsigned long) MagickMin(QuantumDepth,16);        /*Word type cell */
       ldblk=(long) (2 * mat_header.SizeX);
       if ((mat_header.StructureFlag && FLAG_COMPLEX) != 0)
         goto MATLAB_KO;
@@ -565,7 +572,7 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case 9:
     {
-      image->depth=Min(QuantumDepth,32);        /*double type cell */
+      image->depth=(unsigned long) MagickMin(QuantumDepth,32);        /*double type cell */
       if (sizeof(double) != 8)
         ThrowReaderException(CoderError, "IncompatibleSizeOfDouble");
       if ((mat_header.StructureFlag && FLAG_COMPLEX) != 0)
