@@ -1165,7 +1165,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
               buff=DestroyImage(buff);
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
-          AttachBlob(iptc->blob,profile->datum,profile->length);
+          AttachBlob(iptc->blob,GetStringInfoDatum(profile),GetStringInfoLength(profile));
           result=jpeg_embed(image,buff,iptc);
           blob=DetachBlob(iptc->blob);
           blob=(char *) RelinquishMagickMemory(blob);
@@ -1261,6 +1261,9 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
     }
   if (LocaleCompare(image_info->magick,"IPTC") == 0)
     {
+      register unsigned char
+        *p;
+
       buff=AllocateImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -1297,9 +1300,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       /*
         subtract off the length of the 8BIM stuff.
       */
-      length=profile->length-12;
-      profile->datum[10]=(unsigned char) (length >> 8);
-      profile->datum[11]=(unsigned char) (length & 0xff);
+      length=GetStringInfoLength(profile)-12;
+      p=GetStringInfoDatum(profile);
+      p[10]=(unsigned char) (length >> 8);
+      p[11]=(unsigned char) (length & 0xff);
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       (void) SetImageProfile(image,"iptc",profile);
       profile=DestroyStringInfo(profile);
@@ -2153,7 +2157,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
-      (void) WriteBlob(image,profile->length,profile->datum);
+      (void) WriteBlob(image,GetStringInfoLength(profile),
+        GetStringInfoDatum(profile));
       CloseBlob(image);
       return(MagickTrue);
     }
@@ -2169,8 +2174,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-      info=profile->datum;
-      length=profile->length;
+      info=GetStringInfoDatum(profile);
+      length=GetStringInfoLength(profile);
       length=(unsigned long) GetIPTCStream(&info,(unsigned long) length);
       if (length == 0)
         ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
@@ -2194,7 +2199,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       buff=AllocateImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-      AttachBlob(buff->blob,profile->datum,profile->length);
+      AttachBlob(buff->blob,GetStringInfoDatum(profile),GetStringInfoLength(profile));
       format8BIM(buff,image);
       (void) DetachBlob(buff->blob);
       buff=DestroyImage(buff);
@@ -2216,8 +2221,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       profile=GetImageProfile(image,"iptc");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
-      info=profile->datum;
-      length=profile->length;
+      info=GetStringInfoDatum(profile);
+      length=GetStringInfoLength(profile);
       length=(unsigned long) GetIPTCStream(&info,(unsigned long) length);
       if (length == 0)
         ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
@@ -2249,7 +2254,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
-      (void) WriteBlob(image,profile->length,profile->datum);
+      (void) WriteBlob(image,GetStringInfoLength(profile),GetStringInfoDatum(profile));
       CloseBlob(image);
       return(MagickTrue);
     }
@@ -2265,7 +2270,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
-      (void) WriteBlob(image,profile->length,profile->datum);
+      (void) WriteBlob(image,GetStringInfoLength(profile),GetStringInfoDatum(profile));
       CloseBlob(image);
       return(MagickTrue);
     }
