@@ -199,6 +199,20 @@ static voidpf AcquireZIPMemory(voidpf context,unsigned int items,
 }
 #endif
 
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static inline void PushRunlengthPacket(Image *image,const unsigned char *pixels,
   size_t *length,PixelPacket *pixel,IndexPacket *index)
 {
@@ -762,6 +776,16 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                       MagickOrientationOptions,MagickFalse,options);
                     break;
                   }
+                if (LocaleCompare(keyword,"origin") == 0)
+                  {
+                    char
+                      *geometry;
+
+                    geometry=GetPageGeometry(options);
+                    (void) ParseAbsoluteGeometry(geometry,&image->origin);
+                    geometry=(char *) RelinquishMagickMemory(geometry);
+                    break;
+                  }
                 (void) SetImageProperty(image,keyword,options);
                 break;
               }
@@ -858,7 +882,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"ticks-per-second") == 0)
                   {
-                    image->ticks_per_second=(unsigned long) atol(options);
+                    image->ticks_per_second=atol(options);
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -970,7 +994,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         while (name != (const char *) NULL)
         {
           profile=GetImageProfile(image,name);
-          count=ReadBlob(image,profile->length,profile->datum);
+          if (profile != (StringInfo *) NULL)
+            {
+              register unsigned char
+                *p;
+
+              p=GetStringInfoDatum(profile);
+              count=ReadBlob(image,GetStringInfoLength(profile),p);
+            }
           name=(const char *) GetNextValueInLinkedList(profiles);
         }
         profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
@@ -1076,7 +1107,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
       packet_size++;
     length=packet_size*image->columns;
     pixels=(unsigned char *) AcquireMagickMemory(length*sizeof(*pixels));
-    length=(size_t) Max(BZipMaxExtent(packet_size*image->columns),
+    length=MagickMax(BZipMaxExtent(packet_size*image->columns),
       ZipMaxExtent(packet_size*image->columns));
     compress_pixels=(unsigned char *) AcquireMagickMemory(length*
       sizeof(*compress_pixels));
@@ -1375,7 +1406,6 @@ ModuleExport void UnregisterMIFFImage(void)
 %
 %    o image: The image.
 %
-%
 */
 
 static inline size_t PopRunlengthPacket(Image *image,unsigned char *pixels,
@@ -1630,7 +1660,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
       packet_size+=image->depth/8;
     length=packet_size*image->columns;
     pixels=(unsigned char *) AcquireMagickMemory(length*sizeof(*pixels));
-    length=(size_t) Max(BZipMaxExtent(packet_size*image->columns),
+    length=MagickMax(BZipMaxExtent(packet_size*image->columns),
       ZipMaxExtent(packet_size*image->columns));
     if ((compression == BZipCompression) || (compression == ZipCompression))
       if (length != (size_t) ((unsigned int) length))
@@ -1692,6 +1722,12 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             image->page.x,image->page.y);
           (void) WriteBlobString(image,buffer);
         }
+    if ((image->origin.x != 0) || (image->origin.y != 0))
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,"origin=%+ld%+ld\n",
+          image->origin.x,image->origin.y);
+        (void) WriteBlobString(image,buffer);
+      }
     if ((GetNextImageInList(image) != (Image *) NULL) ||
         (GetPreviousImageInList(image) != (Image *) NULL))
       {
@@ -1795,7 +1831,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
           if (profile != (StringInfo *) NULL)
             {
               (void) FormatMagickString(buffer,MaxTextExtent,
-                "profile-%s=%lu\n",name,(unsigned long) profile->length);
+                "profile-%s=%lu\n",name,(unsigned long) GetStringInfoLength(profile));
               (void) WriteBlobString(image,buffer);
             }
           name=GetNextImageProfile(image);
@@ -1858,7 +1894,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
         while (name != (const char *) NULL)
         {
           profile=GetImageProfile(image,name);
-          (void) WriteBlob(image,profile->length,profile->datum);
+          (void) WriteBlob(image,GetStringInfoLength(profile),GetStringInfoDatum(profile));
           name=GetNextImageProfile(image);
         }
       }
@@ -1963,7 +1999,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               zip_info.zfree=RelinquishZIPMemory;
               zip_info.opaque=(voidpf) NULL;
               code=deflateInit(&zip_info,(int) (image->quality ==
-                UndefinedCompressionQuality ? 7 : Min(image->quality/10,9)));
+                UndefinedCompressionQuality ? 7 : MagickMin(image->quality/10,9)));
               if (code >= 0)
                 status=MagickTrue;
             }
@@ -2015,7 +2051,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               bzip_info.bzfree=RelinquishBZIPMemory;
               bzip_info.opaque=(void *) NULL;
               code=BZ2_bzCompressInit(&bzip_info,(int) (image->quality ==
-                UndefinedCompressionQuality ? 7 : Min(image->quality/10,9)),
+                UndefinedCompressionQuality ? 7 : MagickMin(image->quality/10,9)),
                 (int) image_info->verbose,0);
               if (code >= 0)
                 status=MagickTrue;
