@@ -239,7 +239,6 @@ static MagickBooleanType IsPDF(const unsigned char *magick,const size_t offset)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -423,41 +422,47 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (LocaleNCompare(PDFVersion,command,strlen(PDFVersion)) == 0)
       (void) SetImageProperty(image,"pdf:Version",command);
     count=0;
-    if ((cropbox != MagickFalse) &&
-        (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0))
+    if (cropbox != MagickFalse)
       {
-        /*
-          Note region defined by crop box.
-        */
-        count=(ssize_t) sscanf(command,"CropBox [%lf %lf %lf %lf",
-          &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-        if (count != 4)
-          count=(ssize_t) sscanf(command,"CropBox[%lf %lf %lf %lf",
-            &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-      }
-    if ((trimbox != MagickFalse) &&
-        (LocaleNCompare(TrimBox,command,strlen(TrimBox)) == 0))
-      {
-        /*
-          Note region defined by trim box.
-        */
-        count=(ssize_t) sscanf(command,"TrimBox [%lf %lf %lf %lf",
-          &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-        if (count != 4)
-          count=(ssize_t) sscanf(command,"TrimBox[%lf %lf %lf %lf",
-            &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-      }
-    if (LocaleNCompare(MediaBox,command,strlen(MediaBox)) == 0)
-      {
-        /*
-          Note region defined by media box.
-        */
-        count=(ssize_t) sscanf(command,"MediaBox [%lf %lf %lf %lf",
-          &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-        if (count != 4)
-          count=(ssize_t) sscanf(command,"MediaBox[%lf %lf %lf %lf",
-            &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+        if (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0)
+          {
+            /*
+              Note region defined by crop box.
+            */
+            count=(ssize_t) sscanf(command,"CropBox [%lf %lf %lf %lf",
+              &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+            if (count != 4)
+              count=(ssize_t) sscanf(command,"CropBox[%lf %lf %lf %lf",
+                &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+          }
       }
+    else
+      if (trimbox != MagickFalse)
+        {
+          if (LocaleNCompare(TrimBox,command,strlen(TrimBox)) == 0)
+            {
+              /*
+                Note region defined by trim box.
+              */
+              count=(ssize_t) sscanf(command,"TrimBox [%lf %lf %lf %lf",
+                &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+              if (count != 4)
+                count=(ssize_t) sscanf(command,"TrimBox[%lf %lf %lf %lf",
+                  &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+            }
+        }
+      else
+        if (LocaleNCompare(MediaBox,command,strlen(MediaBox)) == 0)
+          {
+            /*
+              Note region defined by media box.
+            */
+            count=(ssize_t) sscanf(command,"MediaBox [%lf %lf %lf %lf",
+              &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+            if (count != 4)
+              count=(ssize_t) sscanf(command,"MediaBox[%lf %lf %lf %lf",
+                &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+          }
     if (count != 4)
       continue;
     if ((bounds.x2 > hires_bounds.x2) && (bounds.y2 > hires_bounds.y2))
@@ -679,9 +684,22 @@ ModuleExport void UnregisterPDFImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static char *EscapeParenthesis(const char *text)
 {
   register char
@@ -698,7 +716,7 @@ static char *EscapeParenthesis(const char *text)
 
   escapes=0;
   p=buffer;
-  for (i=0; i < Min((long) strlen(text),(long) (MaxTextExtent-escapes-1)); i++)
+  for (i=0; i < (long) MagickMin(strlen(text),(MaxTextExtent-escapes-1)); i++)
   {
     if ((text[i] == '(') || (text[i] == ')'))
       {
@@ -750,9 +768,6 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     scene,
     *xref;
 
-  MagickRealType
-    version;
-
   MagickSizeType
     number_pixels;
 
@@ -799,7 +814,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     object,
     pages_id,
     root_id,
-    text_size;
+    text_size,
+    version;
 
   /*
     Open output image file.
@@ -824,15 +840,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     Write Info object.
   */
   object=0;
-  version=1.3f;
+  version=3;
   for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
     if (next->compression == JPEG2000Compression)
-      version=Max(version,1.5);
+      version=(unsigned long) MagickMax(version,5);
     else
       if (next->matte != MagickFalse)
-        version=Max(version,1.4);
-  (void) FormatMagickString(buffer,MaxTextExtent,"%%PDF-%.1f \n",(double)
-    version);
+        version=(unsigned long) MagickMax(version,4);
+  (void) FormatMagickString(buffer,MaxTextExtent,"%%PDF-1.%lu \n",version);
   (void) WriteBlobString(image,buffer);
   xref[object++]=TellBlob(image);
   info_id=object;
