@@ -1216,19 +1216,10 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
     {
       case 1:
       {
-        IndexPacket
-          polarity;
-
         /*
           Convert image to a PBM image.
         */
-        (void) SetImageType(image,PaletteType);
         (void) SetImageType(image,BilevelType);
-        polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
-          (QuantumRange/2) ? 1 : 0);
-        if (image->colors == 2)
-          polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
-             PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
         i=0;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -1239,7 +1230,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
           {
             (void) FormatMagickString(buffer,MaxTextExtent,"%u ",
-              indexes[x] == polarity ? 0x00 : 0x01);
+              PixelIntensity(p) < (QuantumRange/2.0) ? 0x01 : 0x00);
             (void) WriteBlobString(image,buffer);
             i++;
             if (i == 36)
@@ -1247,6 +1238,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
                 (void) WriteBlobByte(image,'\n');
                 i=0;
               }
+            p++;
           }
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
@@ -1359,9 +1351,6 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
       }
       case 4:
       {
-        IndexPacket
-          polarity;
-
         unsigned long
           bit,
           byte;
@@ -1369,14 +1358,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PBM image.
         */
-        (void) SetImageType(image,PaletteType);
         (void) SetImageType(image,BilevelType);
-        polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
-          (QuantumRange/2) ? 1 : 0);
-        if (image->colors == 2)
-          polarity=(IndexPacket)
-            (PixelIntensityToQuantum(&image->colormap[0]) <
-             PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -1388,7 +1370,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
           {
             byte<<=1;
-            if (indexes[x] != polarity)
+            if (PixelIntensity(p) < (QuantumRange/2.0))
               byte|=0x01;
             bit++;
             if (bit == 8)
