@@ -338,6 +338,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   short int
     hex_digits[256];
 
+  size_t
+    length;
+
   ssize_t
     count;
 
@@ -347,7 +350,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long
     columns,
     language_level,
-    length,
     pages,
     rows,
     scene,
@@ -469,21 +471,32 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) sscanf(command,ImageData " %lu %lu",&columns,&rows);
     if (LocaleNCompare(PhotoshopProfile,command,strlen(PhotoshopProfile)) == 0)
       {
+        unsigned long
+          extent;
+
+        unsigned char
+          *p;
+
         /*
           Read image profile.
         */
-        count=(ssize_t) sscanf(command,PhotoshopProfile " %lu",&length);
+        count=(ssize_t) sscanf(command,PhotoshopProfile " %lu",&extent);
         if (count != 1)
           continue;
+        length=extent;
         profile=AcquireStringInfo(length);
+        p=GetStringInfoDatum(profile);
         for (i=0; i < (long) length; i++)
-          profile->datum[i]=(unsigned char) ProfileInteger(image,hex_digits);
+          *p++=(unsigned char) ProfileInteger(image,hex_digits);
         (void) SetImageProfile(image,"IPTC",profile);
         profile=DestroyStringInfo(profile);
         continue;
       }
     if (LocaleNCompare(XMLProfile,command,strlen(XMLProfile)) == 0)
       {
+        unsigned char
+          *p;
+
         /*
           Read image profile.
         */
@@ -491,8 +504,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (count != 1)
           continue;
         profile=AcquireStringInfo(length);
+        p=GetStringInfoDatum(profile);
         for (i=0; i < (long) length; i++)
-          profile->datum[i]=(unsigned char) ReadBlobByte(image);
+          *p++=(unsigned char) ReadBlobByte(image);
         (void) SetImageProfile(image,"XML",profile);
         profile=DestroyStringInfo(profile);
         continue;
@@ -841,19 +855,26 @@ ModuleExport void UnregisterPSImage(void)
 %
 %    o image: The image.
 %
-%
 */
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
 {
 #define WriteRunlengthPacket(image,pixel,length,p) \
 { \
   if ((image->matte != MagickFalse) && (p->opacity == TransparentOpacity)) \
     (void) FormatMagickString(buffer,MaxTextExtent,"ffffff%02lX", \
-      (unsigned long) Min(length,0xff)); \
+      (unsigned long) MagickMin(length,0xff)); \
   else \
     (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02X%02X%02lX", \
       ScaleQuantumToChar(pixel.red),ScaleQuantumToChar(pixel.green), \
-      ScaleQuantumToChar(pixel.blue),(unsigned long) Min(length,0xff)); \
+      ScaleQuantumToChar(pixel.blue),(unsigned long) MagickMin(length,0xff)); \
   (void) WriteBlobString(image,buffer); \
 }
 
@@ -1138,8 +1159,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     type;
 
   IndexPacket
-    index,
-    polarity;
+    index;
 
   long
     j,
@@ -1325,14 +1345,15 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
               Embed Photoshop profile.
             */
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "%%BeginPhotoshop: %lu",(unsigned long) profile->length);
+              "%%BeginPhotoshop: %lu",(unsigned long) GetStringInfoLength(
+              profile));
             (void) WriteBlobString(image,buffer);
-            for (i=0; i < (long) profile->length; i++)
+            for (i=0; i < (long) GetStringInfoLength(profile); i++)
             {
               if ((i % 32) == 0)
                 (void) WriteBlobString(image,"\n% ");
               (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                (unsigned int) (profile->datum[i] & 0xff));
+                (unsigned int) (GetStringInfoDatum(profile)[i] & 0xff));
               (void) WriteBlobString(image,buffer);
             }
             (void) WriteBlobString(image,"\n%EndPhotoshop\n");
@@ -1344,10 +1365,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
               Embed XML profile.
             */
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "%%begin_xml_packet: %lu\n",(unsigned long) profile->length);
+              "%%begin_xml_packet: %lu\n",(unsigned long) GetStringInfoLength(
+              profile));
             (void) WriteBlobString(image,buffer);
-            for (i=0; i < (long) profile->length; i++)
-              (void) WriteBlobByte(image,profile->datum[i]);
+            for (i=0; i < (long) GetStringInfoLength(profile); i++)
+              (void) WriteBlobByte(image,GetStringInfoDatum(profile)[i]);
             (void) WriteBlobString(image,"\n%end_xml_packet\n");
           }
         value=GetImageProperty(image,"Label");
@@ -1394,15 +1416,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as bitmap.
             */
-            type=GetImageType(preview_image,&preview_image->exception);
-            if (type != BilevelType)
-              (void) SetImageType(preview_image,BilevelType);
-            polarity=(IndexPacket) (PixelIntensityToQuantum(
-              &preview_image->colormap[0]) < (QuantumRange/2) ? 1 : 0);
-            if (preview_image->colors == 2)
-              polarity=(IndexPacket) (
-                PixelIntensityToQuantum(&preview_image->colormap[0]) >
-                PixelIntensityToQuantum(&preview_image->colormap[1]) ? 1 : 0);
+            (void) SetImageType(preview_image,BilevelType);
             (void) FormatMagickString(buffer,MaxTextExtent,
               "%%%%BeginPreview: %lu %lu %lu %lu\n%%  ",preview_image->columns,
               preview_image->rows,1L,(((preview_image->columns+7) >> 3)*
@@ -1421,7 +1435,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
               for (x=0; x < (long) preview_image->columns; x++)
               {
                 byte<<=1;
-                if (indexes[x] == polarity)
+                if (PixelIntensity(p) >= (QuantumRange/2.0))
                   byte|=0x01;
                 bit++;
                 if (bit == 8)
@@ -1583,14 +1597,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as bitmap.
             */
-            if (image->storage_class != PseudoClass)
-              (void) SetImageType(image,BilevelType);
-            polarity=(IndexPacket) (PixelIntensityToQuantum(
-              &image->colormap[0]) < (QuantumRange/2) ? 1 : 0);
-            if (image->colors == 2)
-              polarity=(IndexPacket) (
-                PixelIntensityToQuantum(&image->colormap[1]) <
-                PixelIntensityToQuantum(&image->colormap[0]) ? 1 : 0);
+            (void) SetImageType(image,BilevelType);
             count=0;
             for (y=0; y < (long) image->rows; y++)
             {
@@ -1604,7 +1611,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
               for (x=0; x < (long) image->columns; x++)
               {
                 byte<<=1;
-                if (indexes[x] != polarity)
+                if (PixelIntensity(p) >= (QuantumRange/2.0))
                   byte|=0x01;
                 bit++;
                 if (bit == 8)
@@ -1810,7 +1817,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                         {
                           (void) FormatMagickString(buffer,MaxTextExtent,
                             "%02lX%02lX",(unsigned long) index,(unsigned long)
-                            Min(length,0xff));
+                            MagickMin(length,0xff));
                           (void) WriteBlobString(image,buffer);
                           i++;
                           if (i == 18)
@@ -1826,7 +1833,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 (void) FormatMagickString(buffer,MaxTextExtent,"%02lX%02lX",
-                  (unsigned long) index,(unsigned long) Min(length,0xff));
+                  (unsigned long) index,(unsigned long) MagickMin(length,0xff));
                 (void) WriteBlobString(image,buffer);
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
