@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,13 +40,13 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -57,6 +57,8 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/resource_.h"
+#include "magick/property.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -83,8 +85,8 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterPS2Image() adds attributes for the PS2 image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterPS2Image() adds properties for the PS2 image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -104,14 +106,14 @@ ModuleExport void RegisterPS2Image(void)
   entry->encoder=(EncoderHandler *) WritePS2Image;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Level II Encapsulated PostScript");
-  entry->module=AcquireString("PS2");
+  entry->description=ConstantString("Level II Encapsulated PostScript");
+  entry->module=ConstantString("PS2");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS2");
   entry->encoder=(EncoderHandler *) WritePS2Image;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Level II PostScript");
-  entry->module=AcquireString("PS2");
+  entry->description=ConstantString("Level II PostScript");
+  entry->module=ConstantString("PS2");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -330,22 +332,15 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     page_geometry[MaxTextExtent],
     **labels;
 
-  const char
-    **q;
-
   CompressionType
     compression;
 
-  const ImageAttribute
-    *attribute;
+  const char
+    **q,
+    *value;
 
   double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
+    pointsize;
 
   GeometryInfo
     geometry_info;
@@ -368,6 +363,11 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -409,7 +409,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -437,29 +437,29 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     /*
       Scale relative to dots-per-inch.
     */
-    dx_resolution=DefaultResolution;
-    dy_resolution=DefaultResolution;
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
+      resolution.y=resolution.x;
     if (image->x_resolution != 0.0)
-      x_resolution=image->x_resolution;
+      resolution.x=image->x_resolution;
     if (image->y_resolution != 0.0)
-      y_resolution=image->y_resolution;
+      resolution.y=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
         if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+          resolution.y=resolution.x;
       }
     if (image->units == PixelsPerCentimeterResolution)
       {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
+        resolution.x*=2.54;
+        resolution.y*=2.54;
       }
     SetGeometry(image,&geometry);
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
@@ -473,13 +473,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
+    (void) ConcatenateMagickString(page_geometry,">",MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
-    geometry.height=(unsigned long) (y_scale+0.5);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
     if (image->gravity != UndefinedGravity)
@@ -487,20 +488,23 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
         geometry.x=(-page_info.x);
         geometry.y=(long) (media_info.height+page_info.y-image->rows);
       }
+    pointsize=12.0;
+    if (image_info->pointsize != 0.0)
+      pointsize=image_info->pointsize;
     text_size=0;
-    attribute=GetImageAttribute(image,"Label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    value=GetImageProperty(image,"Label");
+    if (value != (const char *) NULL)
+      text_size=(unsigned long) (MultilineCensus(value)*pointsize+12);
     if (page == 1)
       {
         /*
           Output Postscript header.
         */
         if (LocaleCompare(image_info->magick,"PS2") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"%%Creator: (ImageMagick)\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Title: (%s)\n",
@@ -518,8 +522,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
         bounds.x2=(double) geometry.x+geometry.width;
         bounds.y2=(double) geometry.y+geometry.height+text_size;
         if ((image_info->adjoin != MagickFalse) &&
-            (image->next != (Image *) NULL))
-          (void) strcpy(buffer,"%%BoundingBox: (atend)\n");
+            (GetNextImageInList(image) != (Image *) NULL))
+          (void) CopyMagickString(buffer,"%%BoundingBox: (atend)\n",
+            MaxTextExtent);
         else
           {
             (void) FormatMagickString(buffer,MaxTextExtent,
@@ -532,8 +537,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               bounds.x2,bounds.y2);
           }
         (void) WriteBlobString(image,buffer);
-        attribute=GetImageAttribute(image,"Label");
-        if (attribute != (const ImageAttribute *) NULL)
+        value=GetImageProperty(image,"Label");
+        if (value != (const char *) NULL)
           (void) WriteBlobString(image,
             "%%DocumentNeededResources: font Helvetica\n");
         (void) WriteBlobString(image,"%%LanguageLevel: 2\n");
@@ -544,7 +549,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             if (image_info->adjoin == MagickFalse)
-              (void) strcpy(buffer,"%%Pages: 1\n");
+              (void) CopyMagickString(buffer,"%%Pages: 1\n",MaxTextExtent);
             else
               (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
                 (unsigned long) GetImageListLength(image));
@@ -593,15 +598,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           (void) WriteBlobString(image,buffer);
           (void) WriteBlobByte(image,'\n');
         }
-        attribute=GetImageAttribute(image,"Label");
-        if (attribute != (const ImageAttribute *) NULL)
-          for (j=(long) MultilineCensus(attribute->value)-1; j >= 0; j--)
+        value=GetImageProperty(image,"Label");
+        if (value != (const char *) NULL)
+          for (j=(long) MultilineCensus(value)-1; j >= 0; j--)
           {
             (void) WriteBlobString(image,"  /label 512 string def\n");
             (void) WriteBlobString(image,"  currentfile label readline pop\n");
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "  0 y %g add moveto label show pop\n",j*image_info->pointsize+
-              12);
+              "  0 y %g add moveto label show pop\n",j*pointsize+12);
             (void) WriteBlobString(image,buffer);
           }
         for (q=PostscriptEpilog; *q; q++)
@@ -629,8 +633,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       bounds.x2=(double) geometry.x+geometry.width-1;
     if ((double) (geometry.y+(geometry.height+text_size)-1) > bounds.y2)
       bounds.y2=(double) geometry.y+(geometry.height+text_size)-1;
-    attribute=GetImageAttribute(image,"Label");
-    if (attribute != (const ImageAttribute *) NULL)
+    value=GetImageProperty(image,"Label");
+    if (value != (const char *) NULL)
       (void) WriteBlobString(image,"%%PageResources: font Times-Roman\n");
     if (LocaleCompare(image_info->magick,"PS2") != 0)
       (void) WriteBlobString(image,"userdict begin\n");
@@ -645,12 +649,12 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       Output image data.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,x_scale,y_scale,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,pointsize);
     (void) WriteBlobString(image,buffer);
     labels=(char **) NULL;
-    attribute=GetImageAttribute(image,"Label");
-    if (attribute != (const ImageAttribute *) NULL)
-      labels=StringToList(attribute->value);
+    value=GetImageProperty(image,"Label");
+    if (value != (const char *) NULL)
+      labels=StringToList(value);
     if (labels != (char **) NULL)
       {
         for (i=0; labels[i] != (char *) NULL; i++)
@@ -699,7 +703,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           }
           case JPEGCompression:
           {
-            status=JPEGEncodeImage(image_info,image);
+            status=InjectImageBlob(image_info,image,"jpeg");
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
@@ -802,7 +806,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           {
             case JPEGCompression:
             {
-              status=JPEGEncodeImage(image_info,image);
+              status=InjectImageBlob(image_info,image,"jpeg");
               if (status == MagickFalse)
                 ThrowWriterException(CoderError,image->exception.reason);
               break;
@@ -838,9 +842,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                   if ((image->matte != MagickFalse) &&
                       (p->opacity == TransparentOpacity))
                     {
-                      *q++=ScaleQuantumToChar(MaxRGB);
-                      *q++=ScaleQuantumToChar(MaxRGB);
-                      *q++=ScaleQuantumToChar(MaxRGB);
+                      *q++=ScaleQuantumToChar(QuantumRange);
+                      *q++=ScaleQuantumToChar(QuantumRange);
+                      *q++=ScaleQuantumToChar(QuantumRange);
                     }
                   else
                     if (image->colorspace != CMYKColorspace)
@@ -898,9 +902,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                   if ((image->matte != MagickFalse) &&
                       (p->opacity == TransparentOpacity))
                     {
-                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
-                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
-                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
+                      Ascii85Encode(image,ScaleQuantumToChar(QuantumRange));
+                      Ascii85Encode(image,ScaleQuantumToChar(QuantumRange));
+                      Ascii85Encode(image,ScaleQuantumToChar(QuantumRange));
                     }
                   else
                     if (image->colorspace != CMYKColorspace)
@@ -1049,7 +1053,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(image_info->magick,"PS2") != 0)
       (void) WriteBlobString(image,"end\n");
     (void) WriteBlobString(image,"%%PageTrailer\n");
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
