@@ -158,9 +158,22 @@ static MagickBooleanType IsTIFF(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline long MagickMin(const long x,const long y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType ReadProfile(Image *image,const char *name,
   unsigned char *datum,long length)
 {
@@ -700,7 +713,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         packet_size=(size_t) bits_per_sample/8;
         if (image->matte != MagickFalse)
           packet_size*=2;
-        length=Max((size_t) TIFFScanlineSize(tiff),packet_size*
+        length=MagickMax((size_t) TIFFScanlineSize(tiff),packet_size*
           samples_per_pixel*width);
         scanline=(unsigned char *) AcquireMagickMemory(length*
           sizeof(*scanline));
@@ -776,26 +789,26 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             }
           }
         quantum_type=IndexQuantum;
-        quantum_info.pad=(size_t) Max((long) samples_per_pixel-1L,0);
+        quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-1,0);
         if (image->matte != MagickFalse)
           {
             if (image->storage_class != PseudoClass)
               {
                 quantum_type=samples_per_pixel == 1 ? AlphaQuantum :
                   GrayAlphaQuantum;
-                quantum_info.pad=(size_t) Max((long) samples_per_pixel-2L,0);
+                quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-2,0);
               }
             else
               {
                 quantum_type=IndexAlphaQuantum;
-                quantum_info.pad=(size_t) Max((long) samples_per_pixel-2L,0);
+                quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-2,0);
               }
           }
         else
           if (image->storage_class != PseudoClass)
             {
               quantum_type=GrayQuantum;
-              quantum_info.pad=(size_t) Max((long) samples_per_pixel-1L,0);
+              quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-1,0);
             }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -839,21 +852,21 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             TIFFClose(tiff);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        quantum_info.pad=(size_t) Max((long) samples_per_pixel-3L,0);
+        quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-3,0);
         quantum_type=RGBQuantum;
         if (image->matte != MagickFalse)
           {
             quantum_type=RGBAQuantum;
-            quantum_info.pad=(size_t) Max((long) samples_per_pixel-4L,0);
+            quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-4,0);
           }
         if (image->colorspace == CMYKColorspace)
           {
-            quantum_info.pad=(size_t) Max((long) samples_per_pixel-4L,0);
+            quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-4,0);
             quantum_type=CMYKQuantum;
             if (image->matte != MagickFalse)
               {
                 quantum_type=CMYKAQuantum;
-                quantum_info.pad=(size_t) Max((long) samples_per_pixel-5L,0);
+                quantum_info.pad=(size_t) MagickMax((size_t) samples_per_pixel-5,0);
               }
           }
         for (y=0; y < (long) image->rows; y++)
@@ -982,7 +995,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             {
               if (TIFFReadRGBAStrip(tiff,(tstrip_t) y,pixels) == 0)
                 break;
-              i=(long) Min((long) rows_per_strip,(long) image->rows-y);
+              i=(long) MagickMin((long) rows_per_strip,(long) image->rows-y);
             }
           i--;
           p=pixels+image->columns*i;
@@ -1881,7 +1894,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         (void) TIFFSetField(tiff,TIFFTAG_PLANARCONFIG,PLANARCONFIG_SEPARATE);
     rows_per_strip=1;
     if (TIFFScanlineSize(tiff) != 0)
-      rows_per_strip=(unsigned long) Max(TIFFDefaultStripSize(tiff,-1),1);
+      rows_per_strip=(unsigned long) MagickMax((size_t)
+        TIFFDefaultStripSize(tiff,-1),1);
     option=GetImageOption(image_info,"tiff:rows-per-strip");
     if (option != (const char *) NULL)
       rows_per_strip=(unsigned long) strtol(option,(char **) NULL,10);
@@ -1903,7 +1917,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
             ((bits_per_sample == 8) || (bits_per_sample == 16)))
           (void) TIFFSetField(tiff,TIFFTAG_PREDICTOR,2);
         (void) TIFFSetField(tiff,TIFFTAG_ZIPQUALITY,image_info->quality ==
-          UndefinedCompressionQuality ? 7 : Min(image_info->quality/10,9));
+          UndefinedCompressionQuality ? 7 : MagickMin(image_info->quality/10,9));
         break;
       }
       case COMPRESSION_CCITTFAX4:
@@ -1988,13 +2002,13 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           profile=GetImageProfile(image,name);
 #if defined(TIFFTAG_XMLPACKET)
           if (LocaleCompare(name,"xmp") == 0)
-            (void) TIFFSetField(tiff,TIFFTAG_XMLPACKET,(uint32) profile->length,
-              profile->datum);
+            (void) TIFFSetField(tiff,TIFFTAG_XMLPACKET,(uint32)
+              GetStringInfoLength(profile),GetStringInfoDatum(profile));
 #endif
 #if defined(TIFFTAG_ICCPROFILE)
           if (LocaleCompare(name,"icc") == 0)
             (void) TIFFSetField(tiff,TIFFTAG_ICCPROFILE,(uint32)
-              profile->length,profile->datum);
+              GetStringInfoLength(profile),GetStringInfoDatum(profile));
 #endif
           if (LocaleCompare(name,"iptc") == 0)
             {
@@ -2005,13 +2019,15 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 *iptc_profile;
 
               iptc_profile=CloneStringInfo(profile);
-              length=profile->length+4-(profile->length & 0x03);
+              length=GetStringInfoLength(profile)+4-(GetStringInfoLength(
+                profile) & 0x03);
               SetStringInfoLength(iptc_profile,length);
               if (TIFFIsByteSwapped(tiff))
-                TIFFSwabArrayOfLong((uint32 *) iptc_profile->datum,
+                TIFFSwabArrayOfLong((uint32 *) GetStringInfoDatum(iptc_profile),
                   (unsigned long) (length/4));
               (void) TIFFSetField(tiff,TIFFTAG_RICHTIFFIPTC,
-                (uint32) iptc_profile->length/4,iptc_profile->datum);
+                (uint32) GetStringInfoLength(iptc_profile)/4,
+                GetStringInfoDatum(iptc_profile));
               iptc_profile=DestroyStringInfo(iptc_profile);
             }
           if (LocaleCompare(name,"8bim") == 0)
@@ -2020,13 +2036,15 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               uint32
                 length;
 
-              length=(uint32) (profile->length+(profile->length & 0x01));
-              (void) TIFFSetField(tiff,TIFFTAG_PHOTOSHOP,length,profile->datum);
+              length=(uint32) (GetStringInfoLength(profile)+
+                (GetStringInfoLength(profile) & 0x01));
+              (void) TIFFSetField(tiff,TIFFTAG_PHOTOSHOP,length,
+                GetStringInfoDatum(profile));
 #endif
             }
           if (LocaleCompare(name,"tiff:37724") == 0)
-            (void) TIFFSetField(tiff,37724,(uint32) profile->length,
-              profile->datum);
+            (void) TIFFSetField(tiff,37724,(uint32)
+              GetStringInfoLength(profile),GetStringInfoDatum(profile));
           name=GetNextImageProfile(image);
         }
       }
