@@ -316,8 +316,8 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
         "    background color = '%s'",symbol);
     else
       (void) FormatMagickString(buffer,MaxTextExtent,
-        "    color('%s',%s) = '%s'",name,(double)
-        PixelIntensityToQuantum(image->colormap+i) < ((QuantumRange+1.0)/2.0) ?
+        "    color('%s',%s) = '%s'",name,
+        PixelIntensityToQuantum(image->colormap+i) < (QuantumRange/2) ?
         "background" : "foreground",symbol);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"%s",
