@@ -52,7 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -79,7 +79,8 @@ static MagickBooleanType
 %
 %  The format of the IsVICAR method is:
 %
-%      MagickBooleanType IsVICAR(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsVICAR(const unsigned char *magick,
+%        const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -90,14 +91,17 @@ static MagickBooleanType
 %
 %
 */
-static MagickBooleanType IsVICAR(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsVICAR(const unsigned char *magick,
+  const size_t length)
 {
-  if (length < 7)
+  if (length < 14)
     return(MagickFalse);
   if (LocaleNCompare((char *) magick,"LBLSIZE",7) == 0)
     return(MagickTrue);
   if (LocaleNCompare((char *) magick,"NJPL1I",6) == 0)
     return(MagickTrue);
+  if (LocaleNCompare((char *) magick,"PDS_VERSION_ID",14) == 0)
+    return(MagickTrue);
   return(MagickFalse);
 }
 
@@ -153,6 +157,9 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     status,
     value_expected;
 
+  QuantumInfo
+    quantum_info;
+
   register PixelPacket
     *q;
 
@@ -242,17 +249,17 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
           Assign a value to the specified keyword.
         */
         if (LocaleCompare(keyword,"Label_RECORDS") == 0)
-          length=atol(value);
+          length=(ssize_t) atol(value);
         if (LocaleCompare(keyword,"LBLSIZE") == 0)
-          length=atol(value);
+          length=(ssize_t) atol(value);
         if (LocaleCompare(keyword,"RECORD_BYTES") == 0)
-          image->columns= atol(value);
+          image->columns=1UL*atol(value);
         if (LocaleCompare(keyword,"NS") == 0)
-          image->columns= atol(value);
+          image->columns=1UL*atol(value);
         if (LocaleCompare(keyword,"LINES") == 0)
-          image->rows= atol(value);
+          image->rows=1UL*atol(value);
         if (LocaleCompare(keyword,"NL") == 0)
-          image->rows= atol(value);
+          image->rows=1UL*atol(value);
       }
     while (isspace((int) ((unsigned char) c)) != 0)
     {
@@ -260,7 +267,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
       count++;
     }
   }
-  while (count < (long) length)
+  while (count < (ssize_t) length)
   {
     c=ReadBlobByte(image);
     count++;
@@ -278,6 +285,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   /*
     Read VICAR pixels.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   scanline=(unsigned char *) AcquireMagickMemory(image->columns);
   if (scanline == (unsigned char *) NULL)
     ThrowReaderException(CorruptImageError,"UnableToReadImageData");
@@ -287,7 +295,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     if (q == (PixelPacket *) NULL)
       break;
     count=ReadBlob(image,image->columns,scanline);
-    (void) ExportQuantumPixels(image,GrayQuantum,0,scanline);
+    (void) ExportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
     if (SyncImagePixels(image) == MagickFalse)
       break;
     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
@@ -414,6 +422,9 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  QuantumInfo
+    quantum_info;
+
   register const PixelPacket
     *p;
 
@@ -453,13 +464,14 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   /*
     Write VICAR scanline.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   image->depth=8;
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
+    (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
     (void) WriteBlob(image,image->columns,scanline);
     if (image->previous == (Image *) NULL)
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
