@@ -330,7 +330,7 @@ static MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image)
   (void) SetImageInfo(write_info,MagickTrue,&image->exception);
   if (LocaleCompare(write_info->magick,"VID") == 0)
     (void) FormatMagickString(montage_image->filename,MaxTextExtent,
-      "miff:%s",image_info->filename);
+      "miff:%s",write_info->filename);
   status=WriteImage(write_info,montage_image);
   montage_image=DestroyImage(montage_image);
   write_info=DestroyImageInfo(write_info);
