@@ -59,12 +59,12 @@
 #include "magick/quantum.h"
 #include "magick/resize.h"
 #include "magick/resource_.h"
+#include "magick/splay-tree.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/threshold.h"
 #include "magick/utility.h"
 
-
 /*
   Forward declarations.
 */
@@ -92,12 +92,11 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: This string is generally the first few bytes of an image file or
+%      blob.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 {
@@ -133,16 +132,40 @@ static MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 
-static char *ParseColor(char *data)
+static int CompareXPMColor(const void *target,const void *source)
+{ 
+  const char
+    *p,
+    *q;
+  
+  p=(const char *) target;
+  q=(const char *) source;
+  return(strcmp(p,q));
+}
+
+static char *CopyXPMColor(char *destination,const char *source,size_t length)
+{
+  while (length-- && (*source != '\0'))
+    *destination++=(*source++);
+  *destination='\0';
+  return(destination-length);
+}
+
+static char *NextXPMLine(char *p)
+{
+  assert(p != (char*)NULL);
+  p=strchr(p,'\n');
+  if (p != (char *) NULL)
+    p++;
+  return(p);
+}
+
+static char *ParseXPMColor(char *color)
 {
 #define NumberTargets  6
 
-  static const char
-    *targets[NumberTargets] = { "c ", "g ", "g4 ", "m ", "b ", "s " };
-
   register char
     *p,
     *r;
@@ -153,21 +176,24 @@ static char *ParseColor(char *data)
   register long
     i;
 
+  static const char
+    *targets[NumberTargets] = { "c ", "g ", "g4 ", "m ", "b ", "s " };
+
   for (i=0; i < NumberTargets; i++)
   {
-    r=data;
-    for (q=targets[i]; *r != '\0'; r++)
+    p=color;
+    for (q=targets[i]; *p != '\0'; p++)
     {
-      if (*r != *q)
+      if (*p != *q)
         continue;
-      if (isspace((int) ((unsigned char) (*(r-1)))) == 0)
+      if (isspace((int) ((unsigned char) (*(p-1)))) == 0)
         continue;
-      p=r;
+      r=p;
       for ( ; ; )
       {
         if (*q == '\0')
-          return(r);
-        if (*p++ != *q++)
+          return(p);
+        if (*r++ != *q++)
           break;
       }
       q=targets[i];
@@ -180,44 +206,44 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
     key[MaxTextExtent],
-    **keys,
     target[MaxTextExtent],
-    **textlist,
     *xpm_buffer;
 
   Image
     *image;
 
-  ssize_t
-    count;
-
   long
     j,
-    none,
     y;
 
   MagickBooleanType
+    active,
     status;
 
   register char
     *p,
-    *q;
+    *q,
+    *next;
 
   register IndexPacket
     *indexes;
 
   register long
+    i,
     x;
 
   register PixelPacket
     *r;
 
-  register long
-    i;
-
   size_t
     length;
 
+  SplayTreeInfo
+    *xpm_colors;
+
+  ssize_t
+    count;
+
   unsigned long
     width;
 
@@ -247,17 +273,16 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (xpm_buffer != (char *) NULL)
     while (ReadBlobString(image,p) != (char *) NULL)
     {
-      if (*p == '#')
-        if ((p == xpm_buffer) || (*(p-1) == '\n'))
-          continue;
+      if ((*p == '#') && ((p == xpm_buffer) || (*(p-1) == '\n')))
+        continue;
       if ((*p == '}') && (*(p+1) == ';'))
         break;
       p+=strlen(p);
       if ((size_t) (p-xpm_buffer+MaxTextExtent) < length)
         continue;
       length<<=1;
-      xpm_buffer=(char *) ResizeMagickMemory(xpm_buffer,
-        (length+MaxTextExtent)*sizeof(*xpm_buffer));
+      xpm_buffer=(char *) ResizeMagickMemory(xpm_buffer,(length+MaxTextExtent)*
+        sizeof(*xpm_buffer));
       if (xpm_buffer == (char *) NULL)
         break;
       p=xpm_buffer+strlen(xpm_buffer);
@@ -285,66 +310,52 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Remove unquoted characters.
   */
   i=0;
-  for ( ; *p != '\0'; p++)
+  active=MagickFalse;
+  q=xpm_buffer;
+  while (*p != '\0')
   {
-    if (*p != '"')
-      continue;
-    for (q=p+1; *q != '\0'; q++)
-      if (*q == '"')
-        break;
-    (void) CopyMagickString(xpm_buffer+i,p+1,(size_t) (q-p));
-    i+=q-p-1;
-    xpm_buffer[i++]='\n';
-    p=q+1;
+    if (*p++ == '"')
+      {
+        if (active != MagickFalse)
+          *q++='\n';
+        active=active != MagickFalse ? MagickFalse : MagickTrue;
+      }
+    if (active != MagickFalse)
+      *q++=(*p);
   }
-  xpm_buffer[i]='\0';
-  textlist=StringToList(xpm_buffer);
-  xpm_buffer=(char *) RelinquishMagickMemory(xpm_buffer);
-  if (textlist == (char **) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  *q='\0';
   /*
     Initialize image structure.
   */
-  keys=(char **) AcquireMagickMemory((size_t) image->colors*sizeof(*keys));
-  if ((AllocateImageColormap(image,image->colors) == MagickFalse) ||
-      (keys == (char **) NULL))
-    {
-      for (i=0; textlist[i] != (char *) NULL; i++)
-        textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
-      textlist=(char **) RelinquishMagickMemory(textlist);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
+  xpm_colors=NewSplayTree(CompareXPMColor,RelinquishMagickMemory,
+    (void *(*)(void *)) NULL);
+  if (AllocateImageColormap(image,image->colors) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Read image colormap.
   */
   i=1;
-  none=(-1);
-  for (j=0; j < (long) image->colors; j++)
+  next=NextXPMLine(xpm_buffer);
+  for (j=0; (j < (long) image->colors) && (next != (char*) NULL); j++)
   {
-    p=textlist[i++];
-    if (p == (char *) NULL)
-      break;
-    keys[j]=(char *) AcquireMagickMemory((size_t) width+1);
-    if (keys[j] == (char *) NULL)
-      {
-        for (i=0; textlist[i] != (char *) NULL; i++)
-          textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
-        textlist=(char **) RelinquishMagickMemory(textlist);
-        keys=(char **) RelinquishMagickMemory(keys);
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      }
-    (void) CopyMagickString(keys[j],p,(size_t) width+1);
+    p=next;
+    next=NextXPMLine(p);
+    (void) CopyXPMColor(key,p,(size_t) width);
+    status=AddValueToSplayTree(xpm_colors,ConstantString(key),(void *) j);
     /*
       Parse color.
     */
     (void) CopyMagickString(target,"gray",MaxTextExtent);
-    q=ParseColor(p+width);
+    q=ParseXPMColor(p+width);
     if (q != (char *) NULL)
       {
         while ((isspace((int) ((unsigned char) *q)) == 0) && (*q != '\0'))
           q++;
-        (void) CopyMagickString(target,q,MaxTextExtent);
-        q=ParseColor(target);
+        if (next != (char *) NULL)
+          (void) CopyXPMColor(target,q,(size_t) (next-q));
+        else
+          (void) CopyMagickString(target,q,MaxTextExtent);
+        q=ParseXPMColor(target);
         if (q != (char *) NULL)
           *q='\0';
       }
@@ -353,21 +364,13 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         image->storage_class=DirectClass;
         image->matte=MagickTrue;
-        none=j;
-        (void) CopyMagickString(target,"black",MaxTextExtent);
       }
     if (QueryColorDatabase(target,&image->colormap[j],exception) == MagickFalse)
       break;
   }
   if (j < (long) image->colors)
-    {
-      for (i=0; textlist[i] != (char *) NULL; i++)
-        textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
-      textlist=(char **) RelinquishMagickMemory(textlist);
-      ThrowReaderException(CorruptImageError,"CorruptImage");
-    }
+    ThrowReaderException(CorruptImageError,"CorruptImage");
   j=0;
-  key[width]='\0';
   if (image_info->ping == MagickFalse)
     {
       /*
@@ -375,7 +378,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        p=textlist[i++];
+        p=NextXPMLine(p);
         if (p == (char *) NULL)
           break;
         r=SetImagePixels(image,0,y,image->columns,1);
@@ -384,16 +387,11 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          (void) CopyMagickString(key,p,(size_t) width+1);
-          if (strcmp(key,keys[j]) != 0)
-            for (j=0; j < (long) Max(image->colors-1,1); j++)
-              if (strcmp(key,keys[j]) == 0)
-                break;
+          (void) CopyXPMColor(key,p,(size_t) width);
+          j=(long) GetValueFromSplayTree(xpm_colors,key);
           if (image->storage_class == PseudoClass)
             indexes[x]=(IndexPacket) j;
           *r=image->colormap[j];
-          r->opacity=(Quantum)
-            (j == (long) none ? TransparentOpacity : OpaqueOpacity);
           r++;
           p+=width;
         }
@@ -406,12 +404,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Free resources.
   */
-  for (i=0; i < (long) image->colors; i++)
-    keys[i]=(char *) RelinquishMagickMemory(keys[i]);
-  keys=(char **) RelinquishMagickMemory(keys);
-  for (i=0; textlist[i] != (char *) NULL; i++)
-    textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
-  textlist=(char **) RelinquishMagickMemory(textlist);
+  xpm_colors=DestroySplayTree(xpm_colors);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -727,7 +720,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     SetMagickPixelPacket(picon->colormap+i,(IndexPacket *) NULL,&pixel);
     pixel.colorspace=RGBColorspace;
     pixel.depth=8;
-    pixel.opacity=OpaqueOpacity;
+    pixel.opacity=(MagickRealType) OpaqueOpacity;
     (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickFalse,name,
       &image->exception);
     if (LocaleNCompare(name,"rgb",3) == 0)
@@ -778,7 +771,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
       (void) WriteBlobString(image,buffer);
     }
     (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
-      (y == (long) (picon->rows-1) ? "" : ","));
+      y == (long) (picon->rows-1) ? "" : ",");
     (void) WriteBlobString(image,buffer);
     if (QuantumTick(y,picon->rows) != MagickFalse)
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
@@ -967,7 +960,7 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
     SetMagickPixelPacket(image->colormap+i,(IndexPacket *) NULL,&pixel);
     pixel.colorspace=RGBColorspace;
     pixel.depth=8;
-    pixel.opacity=OpaqueOpacity;
+    pixel.opacity=(MagickRealType) OpaqueOpacity;
     (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickFalse,name,
       &image->exception);
     if (LocaleNCompare(name,"rgb",3) == 0)
