@@ -738,8 +738,8 @@ static MagickBooleanType LoadCoderLists(const char *filename,
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    status|=LoadCoderList((const char *) option->datum,option->path,0,
-      exception);
+    status|=LoadCoderList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
