@@ -167,8 +167,11 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          black=(MagickRealType) Min(Min(QuantumRange-q->red,QuantumRange-
-            q->green),QuantumRange-q->blue);
+          black=(MagickRealType) QuantumRange-q->red;
+          if ((QuantumRange-q->green) < black)
+            black=(MagickRealType) QuantumRange-q->green;
+          if ((QuantumRange-q->blue) < black)
+            black=(MagickRealType) QuantumRange-q->blue;
           cyan=(MagickRealType) QuantumRange*(QuantumRange-q->red-black)/
             (QuantumRange-black);
           magenta=(MagickRealType) QuantumRange*(QuantumRange-q->green-black)/
@@ -832,7 +835,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
 %
 %    o colorspace: The colorspace.
 %
-%
 */
 MagickExport MagickBooleanType SetImageColorspace(Image *image,
   const ColorspaceType colorspace)
@@ -1029,14 +1031,11 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         for (x=0; x < (long) image->columns; x++)
         {
           q->red=RoundToQuantum((MagickRealType) (QuantumRange-
-            Min(QuantumRange,QuantumScale*q->red*(QuantumRange-indexes[x])+
-            indexes[x])));
+            QuantumScale*q->red*(QuantumRange-indexes[x])+indexes[x]));
           q->green=RoundToQuantum((MagickRealType) (QuantumRange-
-            Min(QuantumRange,QuantumScale*q->green*(QuantumRange-indexes[x])+
-            indexes[x])));
+            QuantumScale*q->green*(QuantumRange-indexes[x])+indexes[x]));
           q->blue=RoundToQuantum((MagickRealType) (QuantumRange-
-            Min(QuantumRange,QuantumScale*q->blue*(QuantumRange-indexes[x])+
-            indexes[x])));
+            QuantumScale*q->blue*(QuantumRange-indexes[x])+indexes[x]));
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
