@@ -107,6 +107,13 @@
 %
 */
 
+static inline double MagickMin(const double x,const double y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static inline MagickRealType Add(const MagickRealType p,const MagickRealType q)
 {
   MagickRealType
@@ -311,8 +318,8 @@ static inline MagickRealType Difference(const MagickRealType p,
     pixel;
 
   pixel=QuantumScale*(1.0-QuantumScale*alpha)*p+QuantumScale*
-    (1.0-QuantumScale*beta)*q-2.0*Min(QuantumScale*(1.0-QuantumScale*alpha)*p*
-    (1.0-QuantumScale*beta),QuantumScale*(1.0-QuantumScale*beta)*q*
+    (1.0-QuantumScale*beta)*q-2.0*MagickMin(QuantumScale*(1.0-QuantumScale*
+    alpha)*p*(1.0-QuantumScale*beta),QuantumScale*(1.0-QuantumScale*beta)*q*
     (1.0-QuantumScale*alpha));
   return(QuantumRange*pixel);
 }
