@@ -364,8 +364,22 @@ MagickExport void Ascii85Encode(Image *image,const unsigned char code)
 %
 %    o image: The image.
 %
-%
 */
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
 {
 #define HashSize  1021
@@ -625,7 +639,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
       }
     y++;
   }
-  image->rows=(unsigned long) Max(y-3,1);
+  image->rows=(unsigned long) MagickMax((size_t) y-3,1);
   image->compression=FaxCompression;
   /*
     Free decoder memory.
@@ -704,9 +718,6 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   Image
     *huffman_image;
 
-  IndexPacket
-    polarity;
-
   MagickBooleanType
     status;
 
@@ -741,14 +752,19 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   width=image->columns;
   if (LocaleCompare(image_info->magick,"FAX") == 0)
-    width=Max(image->columns,1728);
-  scanline=(unsigned char *) AcquireMagickMemory((size_t) width+1);
+    width=(unsigned long) MagickMax(image->columns,1728);
+  scanline=(unsigned char *) AcquireMagickMemory((size_t) (width+1)*
+    sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
+  (void) ResetMagickMemory(scanline,0,width*sizeof(*scanline));
   huffman_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (huffman_image == (Image *) NULL)
-    return(MagickFalse);
+    {
+      scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      return(MagickFalse);
+    }
   (void) SetImageType(huffman_image,BilevelType);
   byte='\0';
   bit=(unsigned char) 0x80;
@@ -764,17 +780,8 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       OutputBit(1);
     }
   /*
-    Compress runlength encoded to 1D Huffman pixels.
+    Compress to 1D Huffman pixels.
   */
-  polarity=(IndexPacket) (PixelIntensityToQuantum(
-    &huffman_image->colormap[0]) >= (QuantumRange/2) ? 0x00 : 0x01);
-  if (huffman_image->colors == 2)
-    polarity=(IndexPacket)
-      (PixelIntensityToQuantum(&huffman_image->colormap[0]) <
-       PixelIntensityToQuantum(&huffman_image->colormap[1]) ? 0x00 : 0x01);
-  q=scanline;
-  for (i=0; i < (long) width; i++)
-    *q++=(unsigned char) polarity;
   q=scanline;
   for (y=0; y < (long) huffman_image->rows; y++)
   {
@@ -785,9 +792,9 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     indexes=GetIndexes(huffman_image);
     for (x=0; x < (long) huffman_image->columns; x++)
     {
-      *q=(unsigned char)
-        (indexes[x] == polarity ? polarity == 0 : polarity != 0);
-      q++;
+      *q++=(unsigned char) (PixelIntensity(p) >= (QuantumRange/2.0) ?
+        0 : 1);
+      p++;
     }
     /*
       Huffman encode scanline.
@@ -798,7 +805,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       /*
         Output white run.
       */
-      for (runlength=0; ((n > 0) && ((Quantum) *q == polarity)); n--)
+      for (runlength=0; ((n > 0) && (*q == 0)); n--)
       {
         q++;
         runlength++;
@@ -808,18 +815,18 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
           if (runlength < 1792)
             entry=MWTable+((runlength/64)-1);
           else
-            entry=EXTable+(Min(runlength,2560)-1792)/64;
+            entry=EXTable+(MagickMin((size_t) runlength,2560)-1792)/64;
           runlength-=entry->count;
           HuffmanOutputCode(entry);
         }
-      entry=TWTable+Min(runlength,63);
+      entry=TWTable+MagickMin((size_t) runlength,63);
       HuffmanOutputCode(entry);
       if (n != 0)
         {
           /*
             Output black run.
           */
-          for (runlength=0; (((Quantum) *q != polarity) && (n > 0)); n--)
+          for (runlength=0; ((*q != 0) && (n > 0)); n--)
           {
             q++;
             runlength++;
@@ -828,11 +835,11 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
             {
               entry=MBTable+((runlength/64)-1);
               if (runlength >= 1792)
-                entry=EXTable+(Min(runlength,2560)-1792)/64;
+                entry=EXTable+(MagickMin((size_t) runlength,2560)-1792)/64;
               runlength-=entry->count;
               HuffmanOutputCode(entry);
             }
-          entry=TBTable+Min(runlength,63);
+          entry=TBTable+MagickMin((size_t) runlength,63);
           HuffmanOutputCode(entry);
         }
     }
@@ -912,6 +919,9 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   char
     filename[MaxTextExtent];
 
+  FILE
+    *file;
+
   Image
     *huffman_image;
 
@@ -961,14 +971,25 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   DestroyBlob(huffman_image);
   huffman_image->blob=CloneBlobInfo((BlobInfo *) NULL);
   (void) SetImageType(huffman_image,BilevelType);
+  file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(filename);
+  if (unique_file != -1)
+    file=fdopen(unique_file,"wb"); 
+  if ((unique_file == -1) || (file == (FILE *) NULL))
+    {
+      (void) CopyMagickString(image->filename,filename,MaxTextExtent);
+      ThrowFileException(&image->exception,FileOpenError,
+        "UnableToCreateTemporaryFile",image->filename);
+      return(MagickFalse);
+    }
   (void) FormatMagickString(huffman_image->filename,MaxTextExtent,"tiff:%s",
     filename);
   write_info=CloneImageInfo(image_info);
-  SetImageInfoFile(write_info,fdopen(unique_file,"wb"));
+  SetImageInfoFile(write_info,file);
   write_info->compression=Group4Compression;
   status=WriteImage(write_info,huffman_image);
   write_info=DestroyImageInfo(write_info);
+  (void) fclose(file);
   if (status == MagickFalse)
     return(MagickFalse);
   tiff=TIFFOpen(filename,"rb");
@@ -1394,7 +1415,7 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   stream.zfree=(free_func) NULL;
   stream.opaque=(voidpf) NULL;
   status=deflateInit(&stream,(int) (image->quality ==
-    UndefinedCompressionQuality ? 7 : Min(image->quality/10,9)));
+    UndefinedCompressionQuality ? 7 : MagickMin(image->quality/10,9)));
   if (status == Z_OK)
     {
       status=deflate(&stream,Z_FINISH);
