@@ -964,8 +964,8 @@ MagickExport size_t HashStringInfoType(const void *string)
     hash;
 
   GetSignatureInfo(&signature_info);
-  UpdateSignature(&signature_info,((const StringInfo *) string)->datum,
-    ((const StringInfo *) string)->length);
+  UpdateSignature(&signature_info,GetStringInfoDatum((const StringInfo *)
+    string),GetStringInfoLength((const StringInfo *) string));
   FinalizeSignature(&signature_info);
   hash=0;
   for (i=0; i < 8; i++)
