@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image methods.
+  MagickCore image methods.
 */
-#ifndef _MAGICK_IMAGE_H
-#define _MAGICK_IMAGE_H
+#ifndef _MAGICKCORE_IMAGE_H
+#define _MAGICKCORE_IMAGE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -24,17 +24,8 @@ extern "C" {
 
 #include <magick/color.h>
 
-#define ColorMatch(p,q) ((MagickBooleanType) (((p)->red == (q)->red) && \
-  ((p)->green == (q)->green) && ((p)->blue == (q)->blue)))
 #define OpaqueOpacity  ((Quantum) 0UL)
-#define PixelIntensity(pixel) ((MagickRealType) \
-  (0.299*(pixel)->red+0.587*(pixel)->green+0.114*(pixel)->blue+0.5))
-#define PixelIntensityToQuantum(pixel) ((Quantum) \
-  (0.299*(pixel)->red+0.587*(pixel)->green+0.114*(pixel)->blue+0.5))
-#define ScaleColor5to8(x)  (((x) << 3) | ((x) >> 2))
-#define ScaleColor6to8(x)  (((x) << 2) | ((x) >> 4))
-#define TransparentOpacity  ((Quantum) QuantumRange)
-#define UndefinedCompressionQuality  0UL
+#define TransparentOpacity  QuantumRange
 
 typedef enum
 {
@@ -58,15 +49,17 @@ extern MagickExport Image
   *AverageImages(const Image *,ExceptionInfo *),
   *CloneImage(const Image *,const unsigned long,const unsigned long,
     const MagickBooleanType,ExceptionInfo *),
-  *CloneImages(const Image *,const char *,ExceptionInfo *),
   *CombineImages(const Image *,const ChannelType,ExceptionInfo *),
   *DestroyImage(Image *),
   *GetImageClipMask(const Image *,ExceptionInfo *),
+  *GetImageMask(const Image *,ExceptionInfo *),
   *NewMagickImage(const ImageInfo *,const unsigned long,const unsigned long,
     const MagickPixelPacket *),
-  *ReferenceImage(Image *);
+  *ReferenceImage(Image *),
+  *SeparateImages(const Image *,const ChannelType,ExceptionInfo *);
 
 extern MagickExport ImageInfo
+  *AcquireImageInfo(void),
   *CloneImageInfo(const ImageInfo *),
   *DestroyImageInfo(ImageInfo *);
 
@@ -76,11 +69,13 @@ extern MagickExport ImageType
 extern MagickExport IndexPacket
   *GetIndexes(const Image *);
 
+extern MagickExport long
+  InterpretImageFilename(char *,const size_t,const char *,int);
+
 extern MagickExport MagickBooleanType
   AllocateImageColormap(Image *,const unsigned long),
   ClipImage(Image *),
   ClipPathImage(Image *,const char *,const MagickBooleanType),
-  GradientImage(Image *,const PixelPacket *,const PixelPacket *),
   CycleColormapImage(Image *,const long),
   GradientImage(Image *,const PixelPacket *,const PixelPacket *),
   IsTaintImage(const Image *),
@@ -88,10 +83,14 @@ extern MagickExport MagickBooleanType
   ListMagickInfo(FILE *,ExceptionInfo *),
   PlasmaImage(Image *,const SegmentInfo *,unsigned long,unsigned long),
   SeparateImageChannel(Image *,const ChannelType),
+  SetImageBackgroundColor(Image *),
   SetImageClipMask(Image *,const Image *),
+  SetImageExtent(Image *,const unsigned long,const unsigned long),
   SetImageInfo(ImageInfo *,const MagickBooleanType,ExceptionInfo *),
+  SetImageMask(Image *,const Image *),
+  SetImageOpacity(Image *,const Quantum),
+  SetImageStorageClass(Image *,const ClassType),
   SetImageType(Image *,const ImageType),
-  SignatureImage(Image *),
   SortColormapByIntensity(Image *),
   StripImage(Image *),
   SyncImage(Image *),
@@ -109,16 +108,12 @@ extern MagickExport PixelPacket
 
 extern MagickExport void
   AllocateNextImage(const ImageInfo *,Image *),
-  DestroyColorList(void),
   DestroyImagePixels(Image *),
   GetImageException(Image *,ExceptionInfo *),
   GetImageInfo(ImageInfo *),
   ModifyImage(Image **,ExceptionInfo *),
-  RelinquishImageResources(const Image *),
-  SetImageBackgroundColor(Image *),
   SetImageInfoBlob(ImageInfo *,const void *,const size_t),
-  SetImageInfoFile(ImageInfo *,FILE *),
-  SetImageOpacity(Image *,const Quantum);
+  SetImageInfoFile(ImageInfo *,FILE *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
