@@ -944,8 +944,8 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    status|=LoadLocaleList((const char *) option->datum,option->path,locale,0,
-      exception);
+    status|=LoadLocaleList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),locale,0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyLocaleOptions(options);
@@ -956,8 +956,8 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
       option=(const StringInfo *) GetNextValueInLinkedList(options);
       while (option != (const StringInfo *) NULL)
       {
-        status|=LoadLocaleList((const char *) option->datum,option->path,locale,
-          0,exception);
+        status|=LoadLocaleList((const char *) GetStringInfoDatum(option),
+          GetStringInfoPath(option),locale,0,exception);
         option=(const StringInfo *) GetNextValueInLinkedList(options);
       }
       options=DestroyLocaleOptions(options);
