@@ -133,7 +133,6 @@ static MagickBooleanType
 %
 %      DestroyMagick(void)
 %
-%
 */
 MagickExport void DestroyMagick(void)
 {
@@ -219,7 +218,6 @@ MagickExport void DestroyMagickList(void)
 %
 %    o length: The length of the binary string.
 %
-%
 */
 MagickExport const char *GetImageMagick(const unsigned char *magick,
   const size_t length)
@@ -462,7 +460,6 @@ MagickExport MagickBooleanType GetMagickEndianSupport(
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const MagickInfo *GetMagickInfo(const char *name,
   ExceptionInfo *exception)
@@ -783,7 +780,6 @@ MagickExport MagickStatusType GetMagickThreadSupport(
 %
 %    o path: The execution path of the current ImageMagick client.
 %
-%
 */
 
 static SignalHandler *SetMagickSignalHandler(int signal_number,
@@ -838,10 +834,6 @@ static void MagickSignalHandler(int signal_number)
   if (signal_number == SIGHUP)
     exit(signal_number);
 #endif
-#if defined(SIGBREAK)
-  if (signal_number == SIGBREAK)
-    exit(signal_number);
-#endif
 #if defined(SIGINT) && !defined(__WINDOWS__)
   if (signal_number == SIGINT)
     exit(signal_number);
@@ -896,7 +888,6 @@ MagickExport void InitializeMagick(const char *path)
       events=(char *) RelinquishMagickMemory(events);
     }
 #if defined(__WINDOWS__)
-  (void) NTControlHandler();
 #if defined(_DEBUG) && !defined(__BORLANDC__)
   if (IsEventLogging() != MagickFalse)
     {
@@ -931,10 +922,6 @@ MagickExport void InitializeMagick(const char *path)
   if (signal_handlers[SIGABRT] == (SignalHandler *) NULL)
     signal_handlers[SIGABRT]=RegisterMagickSignalHandler(SIGABRT);
 #endif
-#if defined(SIGBREAK)
-  if (signal_handlers[SIGBREAK] == (SignalHandler *) NULL)
-    signal_handlers[SIGBREAK]=RegisterMagickSignalHandler(SIGBREAK);
-#endif
 #if defined(SIGFPE)
   if (signal_handlers[SIGFPE] == (SignalHandler *) NULL)
     signal_handlers[SIGFPE]=RegisterMagickSignalHandler(SIGFPE);
@@ -993,7 +980,6 @@ MagickExport void InitializeMagick(const char *path)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static void *DestroyMagickNode(void *magick_info)
@@ -1162,7 +1148,6 @@ MagickExport MagickBooleanType IsMagickConflict(const char *magick)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListMagickInfo(FILE *file,
   ExceptionInfo *exception)
@@ -1342,7 +1327,6 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
 %    o name: a character string that represents the image format associated
 %      with the MagickInfo structure.
 %
-%
 */
 MagickExport MagickInfo *SetMagickInfo(const char *name)
 {
