@@ -234,7 +234,10 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   */
   module=(MagickBooleanType (*)(Image **,const int,char **))
     lt_dlsym(handle,name);
-  if (module != (MagickBooleanType (*)(Image **,const int,char **)) NULL)
+  if (module == (MagickBooleanType (*)(Image **,const int,char **)) NULL)
+    (void) ThrowMagickException(&(*image)->exception,GetMagickModule(),
+      ModuleError,"UnableToLoadModule","`%s': %s",name,lt_dlerror());
+  else
     {
       if ((*image)->debug != MagickFalse)
         (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
@@ -283,9 +286,6 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
 MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
   ExceptionInfo *exception)
 {
-  register const ModuleInfo
-    *p;
-
   if ((module_list == (SplayTreeInfo *) NULL) ||
       (instantiate_module == MagickFalse))
     if (InitializeModuleList(exception) == MagickFalse)
@@ -302,17 +302,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
       ResetSplayTreeIterator(module_list);
       return((const ModuleInfo *) GetNextValueInSplayTree(module_list));
     }
-  AcquireSemaphoreInfo(&module_semaphore);
-  ResetSplayTreeIterator(module_list);
-  p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
-  while (p != (const ModuleInfo *) NULL)
-  {
-    if (LocaleCompare(p->tag,tag) == 0)
-      break;
-    p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
-  }
-  RelinquishSemaphoreInfo(module_semaphore);
-  return((const ModuleInfo *) p);
+  return((const ModuleInfo *) GetValueFromSplayTree(module_list,tag));
 }
 
 /*
@@ -589,14 +579,14 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     case MagickCoderModule:
     default:
     {
-      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+      (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
         "Searching for coder module file \"%s\" ...",filename);
       module_path=GetEnvironmentValue("MAGICK_CODER_MODULE_PATH");
       break;
     }
     case MagickFilterModule:
     {
-      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+      (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
         "Searching for filter module file \"%s\" ...",filename);
       module_path=GetEnvironmentValue("MAGICK_CODER_FILTER_PATH");
       break;
@@ -619,7 +609,10 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
           (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
         (void) ConcatenateMagickString(path,filename,MaxTextExtent);
         if (IsAccessible(path) != MagickFalse)
-          return(MagickTrue);
+          {
+            module_path=(char *) RelinquishMagickMemory(module_path);
+            return(MagickTrue);
+          }
         p=strchr(p+1,DirectoryListSeparator);
       }
       module_path=(char *) RelinquishMagickMemory(module_path);
@@ -708,7 +701,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       *home;
 
     home=GetEnvironmentValue("MAGICK_HOME");
-		if (home != (char *) NULL)
+    if (home != (char *) NULL)
       {
         /*
           Search MAGICK_HOME.
@@ -807,9 +800,9 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
         */
         (void) FormatMagickString(path,MaxTextExtent,"%s%s%s%s",home,
           *home == '/' ? "/.magick" : "",DirectorySeparator,filename);
+        home=(char *) RelinquishMagickMemory(home);
         if (IsAccessible(path) != MagickFalse)
           return(MagickTrue);
-        home=(char *) RelinquishMagickMemory(home);
       }
   }
   /*
@@ -1054,21 +1047,30 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   module_info=(ModuleInfo *) GetModuleInfo(module,exception);
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
+  AcquireSemaphoreInfo(&module_semaphore);
   (void) CopyMagickString(module_name,module,MaxTextExtent);
   p=GetCoderInfo(module,exception);
   if (p != (CoderInfo *) NULL)
     (void) CopyMagickString(module_name,p->name,MaxTextExtent);
+  if (GetValueFromSplayTree(module_list,module_name) != (void *) NULL)
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      return(MagickTrue);  /* module alread opened, return */
+    }
   /*
-    Find module file.
+    Locate module.
   */
   handle=(ModuleHandle) NULL;
   TagToCoderModuleName(module_name,filename);
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Searching for module \"%s\" using filename \"%s\"",module_name,filename);
   *path='\0';
   length=GetMagickModulePath(filename,MagickCoderModule,path,exception);
   if (length == 0)
-    return(MagickFalse);
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      return(MagickFalse);
+    }
   /*
     Load module
   */
@@ -1077,27 +1079,34 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToLoadModule","`%s': %s",path,lt_dlerror());
+      RelinquishSemaphoreInfo(module_semaphore);
       return(MagickFalse);
     }
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Opening module at path \"%s\"",path);
   /*
-    Add module to module module list.
+    Register module.
   */
   module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
   if (module_info == (ModuleInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateModuleInfo",module_name);
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "UnableToAllocateModuleInfo",module_name);
+    }
   (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
   module_info->path=ConstantString(path);
   module_info->tag=ConstantString(module_name);
-  module_info->signature=MagickSignature;
   module_info->handle=handle;
   (void) time(&module_info->load_time);
+  module_info->signature=MagickSignature;
   if (RegisterModule(module_info,exception) == (ModuleInfo *) NULL)
-    return(MagickFalse);
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      return(MagickFalse);
+    }
   /*
-    Locate and record RegisterFORMATImage function
+    Define RegisterFORMATImage method.
   */
   TagToModuleName(module_name,"Register%sImage",name);
   module_info->register_module=(void (*)(void)) lt_dlsym(handle,name);
@@ -1105,13 +1114,14 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
+      RelinquishSemaphoreInfo(module_semaphore);
       return(MagickFalse);
     }
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
     (void *) module_info->register_module);
   /*
-    Locate and record UnregisterFORMATImage function
+    Define UnregisterFORMATImage method.
   */
   TagToModuleName(module_name,"Unregister%sImage",name);
   module_info->unregister_module=(void (*)(void)) lt_dlsym(handle,name);
@@ -1119,15 +1129,14 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
+      RelinquishSemaphoreInfo(module_semaphore);
       return(MagickFalse);
     }
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
     (void *) module_info->unregister_module);
-  /*
-    Execute RegisterFORMATImage module.
-  */
   module_info->register_module();
+  RelinquishSemaphoreInfo(module_semaphore);
   return(MagickTrue);
 }
 
@@ -1401,9 +1410,6 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
   ExceptionInfo *exception)
 {
-  char
-    name[MaxTextExtent];
-
   /*
     Locate and execute UnregisterFORMATImage module.
   */
@@ -1415,13 +1421,19 @@ static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
   module_info->unregister_module();
   if (lt_dlclose((ModuleHandle) module_info->handle) != 0)
     {
-      (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-        "UnableToCloseModule","`%s': %s",name,lt_dlerror());
+      (void) ThrowMagickException(exception,GetMagickModule(),ModuleWarning,
+        "UnableToCloseModule","`%s': %s",module_info->tag,lt_dlerror());
       return(MagickFalse);
     }
   return(MagickTrue);
 }
 #else
+MagickExport MagickBooleanType ListModuleInfo(FILE *file,
+  ExceptionInfo *exception)
+{
+  return(MagickTrue);
+}
+
 MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   Image **image,const int argc,char **argv)
 {
