@@ -26,7 +26,9 @@ extern "C" {
 
 #define WIN32_LEAN_AND_MEAN
 #define VC_EXTRALEAN
+#define _CRT_SECURE_NO_DEPRECATE
 #include <windows.h>
+#include <wchar.h>
 #include <winuser.h>
 #include <wingdi.h>
 #include <io.h>
@@ -79,7 +81,7 @@ extern "C" {
 #  define fileno  _fileno
 #endif
 #if !defined(ftruncate)
-#  define ftruncate(file,length)  NTFileTruncate(file,length)
+#  define ftruncate(file,length)  NTTruncateFile(file,length)
 #endif
 #if !defined(getcwd)
 #  define getcwd  _getcwd
@@ -87,6 +89,9 @@ extern "C" {
 #if !defined(getpid)
 #  define getpid  _getpid
 #endif
+#if !defined(hypot)
+#  define hypot  _hypot
+#endif
 #if !defined(isatty)
 #  define isatty _isatty
 #endif
@@ -266,7 +271,7 @@ extern MagickExport int
   NTCloseLibrary(void *),
   NTControlHandler(void),
   NTExitLibrary(void),
-  NTFileTruncate(int,const MagickSizeType),
+  NTTruncateFile(int,const MagickSizeType),
   NTGhostscriptDLL(char *,int),
   NTGhostscriptEXE(char *,int),
   NTGhostscriptFonts(char *,int),
