@@ -590,6 +590,14 @@ MagickExport unsigned long GetMagickResourceLimit(const ResourceType type)
 %
 %
 */
+
+static inline long MagickMax(const long x,const long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 MagickExport void InitializeMagickResources(void)
 {
   char
@@ -657,7 +665,8 @@ MagickExport void InitializeMagickResources(void)
 #elif defined(HAVE_GETDTABLESIZE) && defined(POSIX)
   files=getdtablesize();
 #endif
-  (void) SetMagickResourceLimit(FileResource,(unsigned long) Max(3*files/4,64));
+  (void) SetMagickResourceLimit(FileResource,(unsigned long)
+    MagickMax(3L*files/4L,64L));
   limit=GetEnvironmentValue("MAGICK_FILE_LIMIT");
   if (limit != (char *) NULL)
     {
