@@ -345,7 +345,6 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 MagickExport MagickBooleanType GetImageExtrema(const Image *image,
@@ -619,6 +618,23 @@ MagickExport MagickBooleanType GetImageChannelMean(const Image *image,
 %    o exception: Return any errors or warnings in this structure.
 %
 */
+
+static inline unsigned long MagickMax(const unsigned long x,
+  const unsigned long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline unsigned long MagickMin(const unsigned long x,
+  const unsigned long y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
   ExceptionInfo *exception)
 {
@@ -662,7 +678,7 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
   {
     channel_statistics[i].depth=1;
     channel_statistics[i].scale=(unsigned long)
-      QuantumRange/(QuantumRange >> (QuantumDepth-1));
+      (QuantumRange/(QuantumRange >> (QuantumDepth-1)));
     channel_statistics[i].maxima=0;
     channel_statistics[i].minima=QuantumRange;
     channel_statistics[i].mean=0.0;
@@ -796,11 +812,11 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
   }
   for (i=0; i < AllChannels; i++)
   {
-    channel_statistics[AllChannels].depth=Max(
+    channel_statistics[AllChannels].depth=MagickMax(
       channel_statistics[AllChannels].depth,channel_statistics[i].depth);
-    channel_statistics[AllChannels].minima=Min(
+    channel_statistics[AllChannels].minima=(Quantum) MagickMin(
       channel_statistics[AllChannels].minima,channel_statistics[i].minima);
-    channel_statistics[AllChannels].maxima=Max(
+    channel_statistics[AllChannels].maxima=(Quantum) MagickMax(
       channel_statistics[AllChannels].maxima,channel_statistics[i].maxima);
     channel_statistics[AllChannels].mean+=channel_statistics[i].mean;
     channel_statistics[AllChannels].standard_deviation+=
@@ -860,7 +876,7 @@ MagickExport unsigned long GetImageQuantumDepth(const Image *image,
     if (depth <= 16)
       depth=16;
   if (constrain != MagickFalse)
-    depth=Min(depth,QuantumDepth);
+    depth=MagickMin(depth,QuantumDepth);
   return(depth);
 }
 
@@ -922,13 +938,13 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(image->signature == MagickSignature);
   y=(long) image->rows;
-  if (GetImageDepth(image,&image->exception) > Min(depth,QuantumDepth))
+  if (GetImageDepth(image,&image->exception) > MagickMin(depth,QuantumDepth))
     {
       /*
         Scale pixels to desired depth.
       */
-      scale=QuantumRange/(QuantumRange >>
-       (QuantumDepth-Min(depth,QuantumDepth)));
+      scale=QuantumRange/(QuantumRange >> (QuantumDepth-MagickMin(depth,
+        QuantumDepth)));
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
@@ -975,6 +991,6 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
         }
       }
     }
-  image->depth=Min(depth,QuantumDepth);
+  image->depth=MagickMin(depth,QuantumDepth);
   return(y == (long) image->rows ? MagickTrue : MagickFalse);
 }
