@@ -40,7 +40,6 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/exception.h"
@@ -48,6 +47,7 @@
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
+#include "magick/property.h"
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
 #include "magick/signature.h"
@@ -326,6 +326,14 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 %    o source: The source string.
 %
 */
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 MagickExport int CompareStringInfo(const StringInfo *target,
   const StringInfo *source)
 {
@@ -334,10 +342,11 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(target != (StringInfo *) NULL);
-  assert(target->signature == MagickSignature); 
+  assert(target->signature == MagickSignature);
   assert(source != (StringInfo *) NULL);
   assert(source->signature == MagickSignature);
-  status=memcmp(target->datum,source->datum,Min(target->length,source->length));
+  status=memcmp(target->datum,source->datum,MagickMin(target->length,
+    source->length));
   if (status != 0)
     return(status);
   if (target->length == source->length)
@@ -590,7 +599,8 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
       (void) MagickSeek(file,0,SEEK_SET);
       for (i=0; i < length; i+=count)
       {
-        count=read(file,string+i,(size_t) Min(length-i,(size_t) SSIZE_MAX));
+        count=read(file,string+i,(size_t) MagickMin(length-i,(size_t)
+          SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -1065,9 +1075,14 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
 MagickExport long FormatMagickStringList(char *string,const size_t length,
   const char *format,va_list operands)
 {
+  char
+    *locale;
+
   int
     n;
 
+  locale=ConstantString(setlocale(LC_ALL,(const char *) NULL));
+  (void) setlocale(LC_ALL,"C");
 #if defined(HAVE_VSNPRINTF)
   n=vsnprintf(string,length,format,operands);
 #else
@@ -1075,6 +1090,8 @@ MagickExport long FormatMagickStringList(char *string,const size_t length,
 #endif
   if (n < 0)
     string[length-1]='\0';
+  (void) setlocale(LC_ALL,locale);
+  locale=(char *) RelinquishMagickMemory(locale);
   return((long) n);
 }
 
@@ -1188,6 +1205,93 @@ MagickExport char *GetEnvironmentValue(const char *name)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t S t r i n g I n f o D a t u m                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetStringInfoDatum() returns the datum associated with the string.
+%
+%  The format of the GetStringInfoDatum method is:
+%
+%      unsigned char *GetStringInfoDatum(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+*/
+MagickExport unsigned char *GetStringInfoDatum(const StringInfo *string_info)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickSignature);
+  return(string_info->datum);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t S t r i n g I n f o L e n g t h                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetStringInfoLength() returns the string length.
+%
+%  The format of the GetStringInfoLength method is:
+%
+%      size_t GetStringInfoLength(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+*/
+MagickExport size_t GetStringInfoLength(const StringInfo *string_info)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickSignature);
+  return(string_info->length);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t S t r i n g I n f o P a t h                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetStringInfoPath() returns the path associated with the string.
+%
+%  The format of the GetStringInfoPath method is:
+%
+%      const char *GetStringInfoPath(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+*/
+MagickExport const char *GetStringInfoPath(const StringInfo *string_info)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickSignature);
+  return(string_info->path);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   L o c a l e C o m p a r e                                                 %
 %                                                                             %
 %                                                                             %
@@ -1443,7 +1547,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
   for (i=0; i < string_info->length; i+=0x14)
   {
     (void) fprintf(file,"0x%08lx: ",(unsigned long) (0x14*i));
-    for (j=1; j <= (size_t) Min(string_info->length-i,0x14); j++)
+    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
     {
       (void) fprintf(file,"%02lx",(unsigned long) (*(p+j)) & 0xff);
       if ((j % 0x04) == 0)
@@ -1457,7 +1561,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
         (void) fputc(' ',file);
     }
     (void) fputc(' ',file);
-    for (j=1; j <= (size_t) Min(string_info->length-i,0x14); j++)
+    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
     {
       if (isprint((int) ((unsigned char) *p)) != 0)
         (void) fputc(*p,file);
@@ -1532,7 +1636,7 @@ MagickExport void SetStringInfo(StringInfo *string_info,
   assert(source != (StringInfo *) NULL);
   assert(source->signature == MagickSignature);
   (void) ResetMagickMemory(string_info->datum,0,string_info->length);
-  (void) CopyMagickMemory(string_info->datum,source->datum,Min(
+  (void) CopyMagickMemory(string_info->datum,source->datum,MagickMin(
     string_info->length,source->length));
 }
 
@@ -1630,6 +1734,39 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S e t S t r i n g I n f o D a t u m                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetStringInfoPath() sets the path associated with the string.
+%
+%  The format of the SetStringInfoPath method is:
+%
+%      void SetStringInfoPath(StringInfo *string_info,const char *path)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: The string info.
+%
+%    o path: The path.
+%
+*/
+MagickExport void SetStringInfoPath(StringInfo *string_info,const char *path)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickSignature);
+  assert(path != (const char *) NULL);
+  (void) CopyMagickString(string_info->path,path,MaxTextExtent);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S p l i t S t r i n g I n f o                                             %
 %                                                                             %
 %                                                                             %
@@ -2066,7 +2203,7 @@ MagickExport char **StringToList(const char *text)
             "UnableToConvertText",text);
         (void) FormatMagickString(textlist[i],MaxTextExtent,"0x%08lx: ",0x14*i);
         q=textlist[i]+strlen(textlist[i]);
-        for (j=1; j <= (long) Min(strlen(p),0x14); j++)
+        for (j=1; j <= (long) MagickMin(strlen(p),0x14); j++)
         {
           (void) FormatMagickString(hex_string,MaxTextExtent,"%02x",*(p+j));
           (void) CopyMagickString(q,hex_string,MaxTextExtent);
@@ -2082,7 +2219,7 @@ MagickExport char **StringToList(const char *text)
             *q++=' ';
         }
         *q++=' ';
-        for (j=1; j <= (long) Min(strlen(p),0x14); j++)
+        for (j=1; j <= (long) MagickMin(strlen(p),0x14); j++)
         {
           if (isprint((int) ((unsigned char) *p)) != 0)
             *q++=(*p);
