@@ -772,10 +772,10 @@ MagickExport ThresholdMap *GetThresholdMap(const char *map_id,
 
   map=(ThresholdMap *)NULL;
   options=GetConfigureOptions(ThresholdsFilename,exception);
-  while ( ( option=(const StringInfo *) GetNextValueInLinkedList(options) )
+  while (( option=(const StringInfo *) GetNextValueInLinkedList(options) )
           != (const StringInfo *) NULL && map == (ThresholdMap *)NULL )
-    map = GetThresholdMapFile((const char *)option->datum,option->path,map_id,
-      exception);
+    map=GetThresholdMapFile((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),map_id,exception);
   options=DestroyConfigureOptions(options);
   return(map);
 }
@@ -908,9 +908,9 @@ MagickExport MagickBooleanType ListThresholdMaps(FILE *file,
   while ( ( option=(const StringInfo *) GetNextValueInLinkedList(options) )
           != (const StringInfo *) NULL)
   {
-    (void) fprintf(file, "\nPATH: %s\n\n", option->path);
-    status|=ListThresholdMapFile(file,(const char *)option->datum,option->path,
-      exception);
+    (void) fprintf(file,"\nPATH: %s\n\n",GetStringInfoPath(option));
+    status|=ListThresholdMapFile(file,(const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),exception);
   }
   options=DestroyConfigureOptions(options);
   return(status != 0 ? MagickTrue : MagickFalse);
@@ -1096,7 +1096,7 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
     char *p;
 
     p = strchr(threshold_map,',');
-    if ( p != (char *)NULL && isdigit(*(++p)) )
+    if ( p != (char *)NULL && isdigit((int) ((unsigned char) *(++p))) )
       levels.index = strtol(p, &p, 10);
     else
       levels.index = 2;
