@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -128,7 +128,7 @@ static double ElapsedTime(void)
   struct tms
     timer;
 
-  return((double) times(&timer)/CLK_TCK);
+  return((double) (times(&timer)/CLK_TCK));
 #else
 #if defined(__WINDOWS__)
   return(NTElapsedTime());
