@@ -152,8 +152,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   /*
     Initialize chop image attributes.
   */
-  chop_image=CloneImage(image,image->columns-extent.width,
-    image->rows-extent.height,MagickFalse,exception);
+  chop_image=CloneImage(image,image->columns-extent.width,image->rows-
+    extent.height,MagickTrue,exception);
   if (chop_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -1571,7 +1571,7 @@ MagickExport MagickBooleanType TransformImage(Image **image,
   flags=ParseSizeGeometry(transform_image,image_geometry,&geometry);
   if ((transform_image->columns == geometry.width) &&
       (transform_image->rows == geometry.height))
-    return(MagickFalse);
+    return(MagickTrue);
   resize_image=ZoomImage(transform_image,geometry.width,geometry.height,
     &(*image)->exception);
   if (resize_image == (Image *) NULL)
@@ -1881,6 +1881,20 @@ MagickExport Image *TrimImage(const Image *image,ExceptionInfo *exception)
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   geometry=GetImageBoundingBox(image,exception);
+  if ((geometry.width == 0) || (geometry.height == 0))
+    {
+      Image
+        *crop_image;
+
+      crop_image=CloneImage(image,1,1,MagickTrue,exception);
+      if (crop_image == (Image *) NULL)
+        return((Image *) NULL);
+      crop_image->background_color.opacity=TransparentOpacity;
+      (void) SetImageBackgroundColor(crop_image);
+      crop_image->page.x=(-1);
+      crop_image->page.y=(-1);
+      return(crop_image);
+    }
   geometry.x+=image->page.x;
   geometry.y+=image->page.y;
   return(CropImage(image,&geometry,exception));
