@@ -141,11 +141,15 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
     source_file;
 
   size_t
-    length;
+    length,
+    quantum;
 
   ssize_t
     count;
 
+  struct stat
+    file_info;
+
   unsigned char
     *buffer;
 
@@ -183,8 +187,11 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
       (void) RelinquishUniqueFileResource(destination);
       return(MagickFalse);
     }
-  buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
-    sizeof(*buffer));
+  quantum=(size_t) (fstat(source_file,&file_info) < 0 ? 0 :
+    file_info.st_size);
+  if ((quantum == 0) || (quantum > MagickMaxBufferSize))
+    quantum=MagickMaxBufferSize;
+  buffer=(unsigned char *) AcquireMagickMemory(quantum*sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     {
       (void) close(source_file);
@@ -194,7 +201,7 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
     }
   for (length=0; ; )
   {
-    count=read(source_file,buffer,MagickMaxBufferSize);
+    count=read(source_file,buffer,quantum);
     if (count <= 0)
       break;
     length=(size_t) count;
@@ -306,7 +313,7 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
     i;
 
   size_t
-    max_length;
+    quantum;
 
   unsigned char
     *decode;
@@ -315,8 +322,8 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
   assert(source != (char *) NULL);
   assert(length != (size_t *) NULL);
   *length=0;
-  max_length=3*strlen(source)/4+1;
-  decode=(unsigned char *) AcquireMagickMemory(max_length);
+  quantum=3*strlen(source)/4+1;
+  decode=(unsigned char *) AcquireMagickMemory(quantum*sizeof(*decode));
   if (decode == (unsigned char *) NULL)
     return((unsigned char *) NULL);
   i=0;
@@ -417,7 +424,7 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
       }
     }
   *length=i;
-  assert(*length < max_length);
+  assert(*length < quantum);
   return(decode);
 }
 
@@ -464,7 +471,7 @@ MagickExport char *Base64Encode(const unsigned char *blob,
     i;
 
   size_t
-    max_length,
+    quantum,
     remainder;
 
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
@@ -472,8 +479,8 @@ MagickExport char *Base64Encode(const unsigned char *blob,
   assert(blob_length != 0);
   assert(encode_length != (size_t *) NULL);
   *encode_length=0;
-  max_length=4*blob_length/3+4;
-  encode=(char *) AcquireMagickMemory(max_length);
+  quantum=4*blob_length/3+4;
+  encode=(char *) AcquireMagickMemory(quantum*sizeof(*encode));
   if (encode == (char *) NULL)
     return((char *) NULL);
   i=0;
@@ -508,7 +515,7 @@ MagickExport char *Base64Encode(const unsigned char *blob,
     }
   *encode_length=i;
   encode[i++]='\0';
-  assert(i <= max_length);
+  assert(i <= quantum);
   return(encode);
 }
 
