@@ -23,7 +23,7 @@
 %                                 August 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,23 +46,8 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/list.h"
-#include "magick/memory_.h"
-#include "magick/option.h"
-#include "magick/paint.h"
-#include "magick/quantize.h"
-#include "magick/resize.h"
-#include "magick/resource_.h"
-#include "magick/splay-tree.h"
-#include "magick/string_.h"
-#include "magick/version.h"
-#include "wand/magick-wand.h"
+#include "wand/studio.h"
+#include "wand/MagickWand.h"
 #include "wand/magick-wand-private.h"
 #include "wand/wand.h"
 
@@ -77,6 +62,12 @@
 }
 #define MagickWandId  "MagickWand"
 
+/*
+  Global declarations.
+*/
+static unsigned long
+  id = 0;
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -103,15 +94,15 @@
 WandExport void ClearMagickWand(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->quantize_info=DestroyQuantizeInfo(wand->quantize_info);
   wand->image_info=DestroyImageInfo(wand->image_info);
   wand->images=DestroyImageList(wand->images);
-  DestroyExceptionInfo(&wand->exception);
+  (void) DestroyExceptionInfo(&wand->exception);
   wand->quantize_info=CloneQuantizeInfo((QuantizeInfo *) NULL);
-  wand->image_info=CloneImageInfo((ImageInfo *) NULL);
+  wand->image_info=AcquireImageInfo();
   GetExceptionInfo(&wand->exception);
   wand->debug=IsEventLogging();
 }
@@ -145,13 +136,20 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
     *clone_wand;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   clone_wand=(MagickWand *) AcquireMagickMemory(sizeof(*clone_wand));
   if (clone_wand == (MagickWand *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(clone_wand,0,sizeof(*clone_wand));
   clone_wand->id=AcquireWandId();
   (void) FormatMagickString(clone_wand->name,MaxTextExtent,"MagickWand-%lu",
@@ -164,7 +162,7 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
   clone_wand->debug=IsEventLogging();
   if (clone_wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",clone_wand->name);
-  clone_wand->signature=MagickSignature;
+  clone_wand->signature=WandSignature;
   return(clone_wand);
 }
 
@@ -194,15 +192,15 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
 WandExport MagickWand *DestroyMagickWand(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->quantize_info=DestroyQuantizeInfo(wand->quantize_info);
   wand->image_info=DestroyImageInfo(wand->image_info);
   wand->images=DestroyImageList(wand->images);
-  DestroyExceptionInfo(&wand->exception);
+  (void) DestroyExceptionInfo(&wand->exception);
   RelinquishWandId(wand->id);
-  wand->signature=(~MagickSignature);
+  wand->signature=(~WandSignature);
   wand=(MagickWand *) RelinquishMagickMemory(wand);
   return(wand);
 }
@@ -233,7 +231,7 @@ WandExport MagickBooleanType IsMagickWand(const MagickWand *wand)
 {
   if (wand == (const MagickWand *) NULL)
     return(MagickFalse);
-  if (wand->signature != MagickSignature)
+  if (wand->signature != WandSignature)
     return(MagickFalse);
   if (LocaleNCompare(wand->name,MagickWandId,strlen(MagickWandId)) != 0)
     return(MagickFalse);
@@ -245,6 +243,76 @@ WandExport MagickBooleanType IsMagickWand(const MagickWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k C l e a r E x c e p t i o n                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickClearException() clear any exceptions associated with the wand.
+%
+%  The format of the MagickClearException method is:
+%
+%      MagickBooleanType MagickClearException(MagickWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport MagickBooleanType MagickClearException(MagickWand *wand)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  ClearMagickException(&wand->exception);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k G e t I t e r a t o r I n d e x                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetIteratorIndex() returns the position of the iterator in the image
+%  list.
+%
+%  The format of the MagickGetIteratorIndex method is:
+%
+%      MagickBooleanType MagickGetIteratorIndex(MagickWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport long MagickGetIteratorIndex(MagickWand *wand)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if (wand->images == (Image *) NULL)
+    {
+      (void) ThrowMagickException(&wand->exception,GetMagickModule(),WandError,
+        "ContainsNoIterators","`%s'",wand->name);
+      return(-1);
+    }
+  return(GetImageIndexInList(wand->images));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k Q u e r y C o n f i g u r e O p t i o n                       %
 %                                                                             %
 %                                                                             %
@@ -279,7 +347,7 @@ WandExport char *MagickQueryConfigureOption(const char *option)
 
   GetExceptionInfo(&exception);
   configure_info=GetConfigureInfoList(option,&number_options,&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   if (configure_info == (const ConfigureInfo **) NULL)
     return((char *) NULL);
   value=AcquireString(configure_info[0]->value);
@@ -327,7 +395,7 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
 
   GetExceptionInfo(&exception);
   options=GetConfigureList(pattern,number_options,&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   return(options);
 }
 
@@ -342,7 +410,7 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickQueryFontMetrics() returns a 7 element array representing the
+%  MagickQueryFontMetrics() returns a 13 element array representing the
 %  following font metrics:
 %
 %    Element Description
@@ -354,6 +422,12 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
 %          4 text width
 %          5 text height
 %          6 maximum horizontal advance
+%          7 bounding box: x1
+%          8 bounding box: y1
+%          9 bounding box: x2
+%         10 bounding box: y2
+%         11 origin: x
+%         12 origin: y
 %
 %  The format of the MagickQueryFontMetrics method is:
 %
@@ -385,17 +459,17 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
     metrics;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   assert(drawing_wand != (const DrawingWand *) NULL);
   if (wand->images == (Image *) NULL)
     {
       (void) ThrowMagickException(&wand->exception,GetMagickModule(),WandError,
-        "WandContainsNoImages","`%s'",wand->name);
+        "ContainsNoImages","`%s'",wand->name);
       return((double *) NULL);
     }
-  font_metrics=(double *) AcquireMagickMemory(7*sizeof(*font_metrics));
+  font_metrics=(double *) AcquireMagickMemory(13*sizeof(*font_metrics));
   if (font_metrics == (double *) NULL)
     return((double *) NULL);
   draw_info=PeekDrawingWand(drawing_wand);
@@ -421,6 +495,12 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
   font_metrics[4]=metrics.width;
   font_metrics[5]=metrics.height;
   font_metrics[6]=metrics.max_advance;
+  font_metrics[7]=metrics.bounds.x1;
+  font_metrics[8]=metrics.bounds.y1;
+  font_metrics[9]=metrics.bounds.x2;
+  font_metrics[10]=metrics.bounds.y2;
+  font_metrics[11]=metrics.origin.x;
+  font_metrics[12]=metrics.origin.y;
   return(font_metrics);
 }
 
@@ -435,7 +515,7 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickQueryMultilineFontMetrics() returns a 7 element array representing the
+%  MagickQueryMultilineFontMetrics() returns a 13 element array representing the
 %  following font metrics:
 %
 %    Element Description
@@ -447,6 +527,12 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
 %          4 text width
 %          5 text height
 %          6 maximum horizontal advance
+%          7 bounding box: x1
+%          8 bounding box: y1
+%          9 bounding box: x2
+%         10 bounding box: y2
+%         11 origin: x
+%         12 origin: y
 %
 %  This method is like MagickQueryFontMetrics() but it returns the maximum text
 %  width and height for multiple lines of text.
@@ -481,14 +567,14 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
     metrics;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   assert(drawing_wand != (const DrawingWand *) NULL);
   if (wand->images == (Image *) NULL)
     {
       (void) ThrowMagickException(&wand->exception,GetMagickModule(),WandError,
-        "WandContainsNoImages","`%s'",wand->name);
+        "ContainsNoImages","`%s'",wand->name);
       return((double *) NULL);
     }
   font_metrics=(double *) AcquireMagickMemory(7*sizeof(*font_metrics));
@@ -517,6 +603,12 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
   font_metrics[4]=metrics.width;
   font_metrics[5]=metrics.height;
   font_metrics[6]=metrics.max_advance;
+  font_metrics[7]=metrics.bounds.x1;
+  font_metrics[8]=metrics.bounds.y1;
+  font_metrics[9]=metrics.bounds.x2;
+  font_metrics[10]=metrics.bounds.y2;
+  font_metrics[11]=metrics.origin.x;
+  font_metrics[12]=metrics.origin.y;
   return(font_metrics);
 }
 
@@ -557,7 +649,7 @@ WandExport char **MagickQueryFonts(const char *pattern,
 
   GetExceptionInfo(&exception);
   fonts=GetTypeList(pattern,number_fonts,&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   return(fonts);
 }
 
@@ -600,7 +692,7 @@ WandExport char **MagickQueryFormats(const char *pattern,
 
   GetExceptionInfo(&exception);
   formats=GetMagickList(pattern,number_formats,&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   return(formats);
 }
 
@@ -631,7 +723,6 @@ WandExport char **MagickQueryFormats(const char *pattern,
 WandExport void *MagickRelinquishMemory(void *memory)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(memory != (void *) NULL);
   return(RelinquishMagickMemory(memory));
 }
 
@@ -662,10 +753,11 @@ WandExport void *MagickRelinquishMemory(void *memory)
 WandExport void MagickResetIterator(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->iterator=MagickTrue;
+  wand->active=MagickFalse;
+  wand->pend=MagickTrue;
   wand->images=GetFirstImageInList(wand->images);
 }
 
@@ -694,12 +786,63 @@ WandExport void MagickResetIterator(MagickWand *wand)
 WandExport void MagickSetFirstIterator(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->iterator=MagickFalse;
+  wand->active=MagickTrue;
+  wand->pend=MagickFalse;
   wand->images=GetFirstImageInList(wand->images);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k S e t I t e r a t o r I n d e x                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetIteratorIndex() set the iterator to the position in the image list
+%  specified with the index parameter.
+%
+%  The format of the MagickSetIteratorIndex method is:
+%
+%      MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
+%        const long index)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o index: The scene number.
+%
+*/
+WandExport MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
+  const long index)
+{
+  Image
+    *image;
+
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if (wand->images == (Image *) NULL)
+    return(MagickFalse);
+  image=GetImageFromList(wand->images,index);
+  if (image == (Image *) NULL)
+    {
+      InheritException(&wand->exception,&wand->images->exception);
+      return(MagickFalse);
+    }
+  wand->active=MagickTrue;
+  wand->pend=MagickFalse;
+  wand->images=image;
+  return(MagickTrue);
+}
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -725,10 +868,11 @@ WandExport void MagickSetFirstIterator(MagickWand *wand)
 WandExport void MagickSetLastIterator(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->iterator=MagickFalse;
+  wand->active=MagickFalse;
+  wand->pend=MagickTrue;
   wand->images=GetLastImageInList(wand->images);
 }
 
@@ -737,6 +881,52 @@ WandExport void MagickSetLastIterator(MagickWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k W a n d G e n e s i s                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickWandGenesis() initializes the MagickWand environment.
+%
+%  The format of the MaickWandGenesis method is:
+%
+%      void MaickWandGenesis(void)
+%
+*/
+WandExport void MagickWandGenesis(void)
+{
+  id=AcquireWandId();
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k W a n d T e r m i n u s                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickWandTerminus() initializes the MagickWand environment.
+%
+%  The format of the MaickWandTerminus method is:
+%
+%      void MaickWandTerminus(void)
+%
+*/
+WandExport void MagickWandTerminus(void)
+{
+  RelinquishWandId(id);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   N e w M a g i c k W a n d                                                 %
 %                                                                             %
 %                                                                             %
@@ -747,8 +937,7 @@ WandExport void MagickSetLastIterator(MagickWand *wand)
 %
 %  The format of the NewMagickWand method is:
 %
-%      MagickWand NewMagickWand(void)
-%
+%      MagickWand *NewMagickWand(void)
 %
 */
 WandExport MagickWand *NewMagickWand(void)
@@ -765,23 +954,30 @@ WandExport MagickWand *NewMagickWand(void)
   depth=QuantumDepth;
   quantum=GetMagickQuantumDepth(&depth);
   if (depth != QuantumDepth)
-    ThrowMagickFatalException(WandError,"WandQuantumDepthMismatch",quantum);
+    ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   wand=(MagickWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (MagickWand *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
   (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",MagickWandId,
     wand->id);
   GetExceptionInfo(&wand->exception);
-  wand->image_info=CloneImageInfo((ImageInfo *) NULL);
+  wand->image_info=AcquireImageInfo();
   wand->quantize_info=CloneQuantizeInfo((QuantizeInfo *) NULL);
   wand->images=NewImageList();
   wand->debug=IsEventLogging();
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  wand->signature=MagickSignature;
+  wand->signature=WandSignature;
   return(wand);
 }
 
